package control.factory;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;

import javax.servlet.http.HttpServletRequest;

import com.oreilly.servlet.MultipartRequest;

import control.utility.ApplicationGlobals;
import entity.Dictionary;

/*
 * **************************************************************************
 *  <p>RequestFactory factory class
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */
 
public class RequestFactory {
	
	private HttpServletRequest _request = null;

	private MultipartRequest _multipartRequest = null;
	
	private boolean _invalidSession = true;
	
	public RequestFactory(HttpServletRequest request){
		_request = request;
		_invalidSession = (!_request.isRequestedSessionIdValid());
	}

	public Dictionary makeDictionary()
	{
		return makeDictionary(false);
	}
	
	public Dictionary makeDictionary(boolean fromMultipart)
	{
		Dictionary dictionary = new Dictionary();
		
		if (_invalidSession)
		{
			Enumeration e=_request.getAttributeNames();
			_makeDictionaryByAttributes(dictionary,e);
		}
		else
		{
			if (fromMultipart)
			{
				_multipartRequest = _buildMultipartRequest(_request);
				Enumeration e = _multipartRequest.getParameterNames();
				_makeDictionaryByParametersFromMultipart(dictionary,e);
			}
			else
			{
				Enumeration e=_request.getParameterNames();
				_makeDictionaryByParameters(dictionary,e);
			}
		}
		
		return dictionary;
	}
	
	public ArrayList makeFiles()	
	{
		ArrayList retFiles = null;
		
		if (_multipartRequest == null)
		{
			_multipartRequest = _buildMultipartRequest(_request);
		}
		
		if (_multipartRequest != null)
		{
			retFiles = _buildFilesFromRequest();
		}
		
		return retFiles;
	}
	
	private void _makeDictionaryByAttributes(Dictionary dictionary, Enumeration e)
	{
		while (e.hasMoreElements()){
			String name = e.nextElement().toString();
			dictionary.setValue(name,_request.getAttribute(name).toString()); 
		}
	}
	
	private void _makeDictionaryByParameters(Dictionary dictionary, Enumeration e)
	{
		while (e.hasMoreElements()){
			String name = e.nextElement().toString();
			dictionary.setValues(name,_request.getParameterValues(name)); 
		}
	}
	
	protected void _makeDictionaryByParametersFromMultipart(Dictionary dictionary, Enumeration e) 
	{
		while (e.hasMoreElements()){
			String name = e.nextElement().toString();
			dictionary.setValues(name,_multipartRequest.getParameterValues(name)); 
		}
	}
	
	private MultipartRequest _buildMultipartRequest(HttpServletRequest request) 
	{	
		MultipartRequest mr=null;
		try {
			mr = new MultipartRequest(request, 
							ApplicationGlobals.MULTIPARTREQUEST_DEFAULT_DIRECTORY, 
								ApplicationGlobals.MULTIPARTREQUEST_DEFAULT_MAX_SIZE);
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return mr;
	}
	
	private ArrayList _buildFilesFromRequest()
	{
		ArrayList vFiles = null;
		
	   //add file parameters...
	   
	   if (_multipartRequest != null)
	   {
		   vFiles = new ArrayList();
		   Enumeration e = _multipartRequest.getFileNames();
	
		   while (e.hasMoreElements())
		   {
				String filename = (String)e.nextElement();
				File f = _multipartRequest.getFile(filename);
				if (f !=null)
				{
					vFiles.add(f);
				}
		   }
	   }
	   
	   return vFiles;
	}
}
