package database;

import java.io.IOException;
import java.io.OutputStream;
import java.sql.Blob;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;

import database.utility.CFiltro;
import database.utility.CFiltroCondicion;
import database.utility.DaoException;
import database.utility.DatabaseDaoDb;
import entity.hibernate.Comentario;
import entity.hibernate.Publicacion;

 
/*
 * **************************************************************************
 * <p>ComentarioDaoDb dao class extends DatabaseDaoDb
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */
 
public class ComentarioDaoDb extends DatabaseDaoDb
{
	private java.sql.Connection _connection;
	private String DATABASE_SELECT = "SELECT * FROM COMENTARIOS"; 
	private String DATABASE_INSERT = "INSERT INTO COMENTARIOS (IdPublicacion, IdComentario, IdUsuarioComentario, FechaComentario, Texto) VALUES (?,?,?,?,?)"; 
	
	public static final String DB_IDPUBLICACION 		= "IdPublicacion";
	public static final String DB_IDCOMENTARIO 			= "IdComentario";
	public static final String DB_IDUSUARIOCOMENTARIO	= "IdUsuarioComentario";
	public static final String DB_FECHACOMENTARIO 		= "FechaComentario";
	public static final String DB_TEXTO 				= "Texto";

	public ComentarioDaoDb (java.sql.Connection connection)
	{
		_connection = connection;   
	}
	
	public Vector consultar(CFiltro filtro, Vector orden) throws DaoException {
		String sentencia = DATABASE_SELECT;
		sentencia += DatabaseDaoDb.construirWhere (filtro);
		sentencia += DatabaseDaoDb.construirOrderBy (orden);		
		ResultSet srs = DatabaseDaoDb.createStatement(_connection, sentencia);
		return _obtenerDatosResultSet (srs);
	}
	
	public Vector consultar(CFiltro filtro) throws DaoException {
		String sentencia = DATABASE_SELECT;
		sentencia += DatabaseDaoDb.construirWhere (filtro);
		ResultSet srs = DatabaseDaoDb.createStatement(_connection, sentencia);
		return _obtenerDatosResultSet (srs);
	}
	
	public Vector consultar() throws DaoException {
		String sentencia = DATABASE_SELECT;
		ResultSet srs = DatabaseDaoDb.createStatement(_connection, sentencia);
		return _obtenerDatosResultSet (srs);
	}
	
	private Vector _obtenerDatosResultSet (ResultSet srs) throws DaoException {
		try {
			Vector retVector = new Vector();
			while (srs.next())	{
				int idPublicacion = srs.getInt(DB_IDPUBLICACION);
				int idComentario = srs.getInt(DB_IDCOMENTARIO);
				int idUsuarioComentario = srs.getInt(DB_IDUSUARIOCOMENTARIO);				
				Date fechaComentario = srs.getTimestamp(DB_FECHACOMENTARIO);
				String texto = srs.getString(DB_TEXTO);

				Comentario comentario = new Comentario (idPublicacion, idComentario, idUsuarioComentario, fechaComentario, texto); 
				retVector.addElement(comentario);
			}
			return retVector;
		  
		} catch(SQLException ex) {
			throw new DaoException(DaoException.EXC_SQLException, ex.getMessage());
		}
	}

	public boolean insert(Comentario comentario) throws DaoException
	{
		boolean retCode = false;
		SimpleDateFormat dtf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");

		PreparedStatement stmt;
		try {
			stmt = _connection.prepareStatement(DATABASE_INSERT);

			stmt.setInt(1, comentario.getIdPublicacion());
			stmt.setNull(2, java.sql.Types.INTEGER);
			stmt.setInt(3, comentario.getIdUsuario());
			stmt.setString(4, dtf.format(comentario.getFechaComentario()));
			if (comentario.getTexto()!=null && !comentario.getTexto().equals("")) stmt.setString(5, comentario.getTexto());
			else stmt.setNull(5, java.sql.Types.VARCHAR);
			
			stmt.execute();
			retCode = true;
				
		} catch(SQLException ex) {
			throw new DaoException(DaoException.EXC_SQLException, ex.getMessage());
		}
		
		return retCode;
	}
	
	public boolean delete(Comentario entity) throws DaoException
	{
		return false;	
	}
	
	public boolean update(Comentario entity) throws DaoException
	{
		return false;
	}
	
}