package database;
 
import java.io.Serializable;
import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Expression;

import control.utility.ApplicationGlobals;
import control.utility.Logger;
import database.utility.CFiltro;
import database.utility.CFiltroCondicion;
import database.utility.DaoException;
import database.utility.FabricaDAODatabase;
import entity.hibernate.Barrio;
import entity.hibernate.Comentario;
import entity.hibernate.Publicacion;
import entity.hibernate.Usuario;
import entity.hibernate.Zona;


/*
 * **************************************************************************
 * <p>SystemDaoDbUtility utility class
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */
 
public class DatabaseDaoDbUtility 
{
	protected Connection _connection = null;
	
	public DatabaseDaoDbUtility()
	{
		FabricaDAODatabase fabricaDAODatabase = new FabricaDAODatabase();
		if (_connection == null) fabricaDAODatabase.createConnection();
		_connection = fabricaDAODatabase.getConnection();
	}
	
	public boolean insertUsuario(Usuario entity)
	{
		if (_connection == null) return false;
								
		boolean retCode = false;
		
		UsuarioDaoDb dao = new UsuarioDaoDb(_connection);
	 
		try
		{
			//recover data from database...
			retCode = dao.insert(entity);
		}
		catch(DaoException e)
		{
			e.printStackTrace();
		}
		
		return retCode;
	}
	
	public boolean updateUsuario(Usuario entity)
	{
		if (_connection == null) return false;
								
		boolean retCode = false;
		
		UsuarioDaoDb dao = new UsuarioDaoDb(_connection);
	 
		try
		{
			//recover data from database...
			
			retCode = dao.update(entity); 
		}
		catch(DaoException e)
		{
			e.printStackTrace();
		}
		
		return retCode;
	}
	
	public Usuario retrieveUser(Integer userId) 
	{
		if (_connection == null) return null;
		
		Usuario user = null;
		
		CFiltroCondicion condicion =
			new CFiltroCondicion(
				UsuarioDaoDb.DB_IDUSUARIO,
				CFiltroCondicion.COMPARADOR_IGUAL,
				userId.toString());
			
		CFiltro filtro = new CFiltro(condicion);
		
		UsuarioDaoDb dao = new UsuarioDaoDb(_connection);
		
		try
		{
			//recover data from database...
			Vector vector = dao.consultar(filtro);
		
			if (vector != null && vector.size()==1) {
				user = (Usuario)vector.get(0);
			}
		}
		catch(DaoException e)
		{
			e.printStackTrace();
		}		
		return user;
	}
		
	public Zona retrieveZona(Integer idZona) 
	{
		if (_connection == null) return null;
		
		Zona zona = null;
		
		CFiltroCondicion condicion =
			new CFiltroCondicion(
				ZonaDaoDb.DB_IDZONA,
				CFiltroCondicion.COMPARADOR_IGUAL,
				idZona.toString());
			
		CFiltro filtro = new CFiltro(condicion);
		
		ZonaDaoDb dao = new ZonaDaoDb(_connection);
		
		try
		{
			//recover data from database...
			Vector vector = dao.consultar(filtro);
		
			if (vector != null && vector.size()==1) {
				zona = (Zona)vector.get(0);
			}
		}
		catch(DaoException e)
		{
			e.printStackTrace();
		}		
		return zona;
	}

	public Barrio retrieveBarrio(Integer idZona, Integer idBarrio) 
	{
		if (_connection == null) return null;
		
		Barrio barrio = null;
		
		CFiltroCondicion condicionZona =
			new CFiltroCondicion(
				BarrioDaoDb.DB_IDZONA,
				CFiltroCondicion.COMPARADOR_IGUAL,
				idZona.toString());

		CFiltroCondicion condicionBarrio =
			new CFiltroCondicion(
				BarrioDaoDb.DB_IDBARRIO,
				CFiltroCondicion.COMPARADOR_IGUAL,
				idBarrio.toString());
			
		CFiltro filtro = new CFiltro(condicionZona, CFiltro.AND, new CFiltro(condicionBarrio));
		
		BarrioDaoDb dao = new BarrioDaoDb(_connection);
		
		try
		{
			//recover data from database...
			Vector vector = dao.consultar(filtro);
		
			if (vector != null && vector.size()==1) {
				barrio = (Barrio)vector.get(0);
			}
		}
		catch(DaoException e)
		{
			e.printStackTrace();
		}		
		return barrio;
	}

	public Publicacion retrievePublicacion(Integer idPublicacion) 
	{
		if (_connection == null) return null;
		
		Publicacion publicacion = null;
		
		CFiltroCondicion condicion =
			new CFiltroCondicion(
				PublicacionDaoDb.DB_IDPUBLICACION,
				CFiltroCondicion.COMPARADOR_IGUAL,
				idPublicacion.toString());
			
		CFiltro filtro = new CFiltro(condicion);
		
		PublicacionDaoDb dao = new PublicacionDaoDb(_connection);
		
		try
		{
			//recover data from database...
			Vector vector = dao.consultar(filtro);
		
			if (vector != null && vector.size()==1) {
				publicacion = (Publicacion)vector.get(0);
				
				_completarPublicacion(publicacion);
			}
		}
		catch(DaoException e)
		{
			e.printStackTrace();
		}		
		return publicacion;
	}

	public Usuario retrieveUser(String email)
	{ 
		Usuario user = null;
	
		if (_connection == null) return null;
								
		CFiltroCondicion condicion =
			new CFiltroCondicion(
				UsuarioDaoDb.DB_EMAIL,
				CFiltroCondicion.COMPARADOR_IGUAL,
				email);
			
		CFiltro filtro = new CFiltro(condicion);
		
		UsuarioDaoDb dao = new UsuarioDaoDb(_connection);
	 
		try
		{
			//recover data from database...
			
			Vector vector = dao.consultar(filtro);
		
			if (vector != null && vector.size()==1) {
				user = (Usuario)vector.get(0);
			}
		}
		catch(DaoException e)
		{
			e.printStackTrace();
		}
		
		return user;
	}
	
	public boolean insertPublicacion(Publicacion entity)
	{
		if (_connection == null) return false;
		
		boolean retCode = false;
		
		PublicacionDaoDb dao = new PublicacionDaoDb(_connection);
	 
		try
		{
			//recover data from database...
			retCode = dao.insert(entity);
		}
		catch(DaoException e)
		{
			e.printStackTrace();
		}
		
		return retCode;	
	}
	
	public List retrievePublicaciones(Integer idUsuario)
	{ 
		List result = null;
	
		if (_connection == null) return null;
								
		CFiltroCondicion condicion =
			new CFiltroCondicion(
				PublicacionDaoDb.DB_IDUSUARIO,
				CFiltroCondicion.COMPARADOR_IGUAL,
				idUsuario.toString());
			
		CFiltro filtro = new CFiltro(condicion);

		PublicacionDaoDb dao = new PublicacionDaoDb(_connection);
	 
		try
		{
			//recover data from database...
			
			result = dao.consultar(filtro); 
			
			Iterator it = result.iterator();
			while (it.hasNext()) {
				_completarPublicacion((Publicacion)it.next());
			}
		}
		catch(DaoException e)
		{
			e.printStackTrace();
		}
		
		return result;
	}	
	
	public List retrieveSeguimientos(Integer idUsuario)
	{ 
		List result = null;
	
		if (_connection == null) return null;
								
		CFiltroCondicion condicion =
			new CFiltroCondicion(
				SeguimientosDaoDb.DB_IDUSUARIO,
				CFiltroCondicion.COMPARADOR_IGUAL,
				idUsuario.toString());
			
		CFiltro filtro = new CFiltro(condicion);

		SeguimientosDaoDb dao = new SeguimientosDaoDb(_connection);
	 
		try
		{
			//recover data from database...
			
			result = dao.consultar(filtro); 
			
			result = _completarSeguimientos(result);
		}
		catch(DaoException e)
		{
			e.printStackTrace();
		}
		
		return result;
	}
	
	public List retrieveZonas()
	{ 
		List result = null;
	
		if (_connection == null) return null;
								
		ZonaDaoDb dao = new ZonaDaoDb(_connection);
	 
		try
		{
			//recover data from database...
			
			result = dao.consultar(); 
		}
		catch(DaoException e)
		{
			e.printStackTrace();
		}
		
		return result;
	}	

	public List retrieveBarrios()
	{ 
		List result = null;
	
		if (_connection == null) return null;
								
		BarrioDaoDb dao = new BarrioDaoDb(_connection);
	 
		try
		{
			//recover data from database...
			
			result = dao.consultar(); 
		}
		catch(DaoException e)
		{
			e.printStackTrace();
		}
		
		return result;
	}	
	
	public List retrieveUsuarios()
	{ 
		List result = null;
	
		if (_connection == null) return null;
								
		UsuarioDaoDb dao = new UsuarioDaoDb(_connection);
	 
		try
		{
			//recover data from database...
			
			result = dao.consultar(); 
		}
		catch(DaoException e)
		{
			e.printStackTrace();
		}
		
		return result;
	}	

	public List retrievePublicaciones(Integer idZona, Integer idBarrio, Integer idUsuario) 
	{ 
		List result = null;
	
		if (_connection == null) return null;
		
		CFiltro filtro = null;
		
		if (idUsuario !=null) {
			CFiltroCondicion condicionUsuario =
				new CFiltroCondicion(
					PublicacionDaoDb.DB_IDUSUARIO,
					CFiltroCondicion.COMPARADOR_IGUAL,
					idUsuario.toString());
			filtro = new CFiltro(condicionUsuario);
		}
		
		if (idZona != null) {
			CFiltroCondicion condicionZona =
				new CFiltroCondicion(
					PublicacionDaoDb.DB_ZONA,
					CFiltroCondicion.COMPARADOR_IGUAL,
					idZona.toString());

			if (idBarrio != null) {
				CFiltroCondicion condicionBarrio =
					new CFiltroCondicion(
						PublicacionDaoDb.DB_BARRIO,
						CFiltroCondicion.COMPARADOR_IGUAL,
						idBarrio.toString());
	
				CFiltro filtroZonaBarrio = new CFiltro(condicionZona, CFiltro.AND, new CFiltro(condicionBarrio));

				if (filtro != null) {
					filtro.addSiguiente(filtroZonaBarrio, CFiltro.OR);
				}
				else {
					filtro = filtroZonaBarrio;
				}
			}
			else {
				if (filtro != null) {
					filtro.addSiguiente(new CFiltro(condicionZona), CFiltro.OR);
				}
				else {
					filtro = new CFiltro(condicionZona);
				}
			}
		}

		PublicacionDaoDb dao = new PublicacionDaoDb(_connection);
	 
		try
		{
			//recover data from database...
			if (filtro != null) {
				result = dao.consultar(filtro);
			}
			else {
				result = dao.consultar();
			}
			
			Iterator it = result.iterator();
			while (it.hasNext()) {
				_completarPublicacion((Publicacion)it.next());
			}
		}
		catch(DaoException e)
		{
			e.printStackTrace();
		}
		
		return result;
	}
	
	public List retrieveComentariosPublicacion(Integer idPublicacion) 
	{ 
		List result = null;
	
		if (_connection == null) return null;
								
		CFiltroCondicion condicion =
			new CFiltroCondicion(
					ComentarioDaoDb.DB_IDPUBLICACION,
				CFiltroCondicion.COMPARADOR_IGUAL,
				idPublicacion.toString());

		CFiltro filtro = new CFiltro(condicion);

		ComentarioDaoDb dao = new ComentarioDaoDb(_connection);
	 
		try
		{
			//recover data from database...
			
			result = dao.consultar(filtro); 
			
			Iterator it = result.iterator();
			while (it.hasNext()) {
				_completarComentario((Comentario)it.next());
			}

		}
		catch(DaoException e)
		{
			e.printStackTrace();
		}
		
		return result;
	}
	
	public boolean insertComentario(Comentario entity)
	{
		if (_connection == null) return false;
		
		boolean retCode = false;
		
		ComentarioDaoDb dao = new ComentarioDaoDb(_connection);
	 
		try
		{
			//insert data from database...
			retCode = dao.insert(entity);
		}
		catch(DaoException e)
		{
			e.printStackTrace();
		}
		
		return retCode;	
	}
	
	public List retrieveComentario()
	{ 
		List result = null;
	
		if (_connection == null) return null;
								
		ComentarioDaoDb dao = new ComentarioDaoDb(_connection);
	 
		try
		{
			//recover data from database...
			
			result = dao.consultar(); 
			
			Iterator it = result.iterator();
			while (it.hasNext()) {
				_completarComentario((Comentario)it.next());
			}
		}
		catch(DaoException e)
		{
			e.printStackTrace();
		}
		
		return result;
	}	
	
	public boolean insertSeguimientos(Usuario user, List bean)
	{
		if (_connection == null) return false;
		
		boolean retCode = true;
		
		SeguimientosDaoDb dao = new SeguimientosDaoDb(_connection);
	 
		try
		{
			Iterator it = bean.iterator();
			while (it.hasNext()) {
				Integer idUsuarioSeguir = new Integer((String)it.next());
				
				//insert data from database...
				retCode = retCode && dao.insert(user.getIdUsuario(), idUsuarioSeguir);
			}
			if (retCode == false) _connection.rollback();
		}
		catch(DaoException e)
		{
			e.printStackTrace();
		} 
		catch (SQLException e) 
		{
			e.printStackTrace();
		}
		
		return retCode;	
	}

	private void _completarPublicacion (Publicacion publicacion)
	{
		Zona zona = retrieveZona(publicacion.getIdZona());
		publicacion.setZona(zona);
		
		Barrio barrio = retrieveBarrio(publicacion.getIdZona(), publicacion.getIdBarrio());
		publicacion.setBarrio(barrio);
		
		Usuario usuario = retrieveUser(publicacion.getIdUsuario());
		publicacion.setUsuario(usuario);
		
		List comentarios = retrieveComentariosPublicacion(publicacion.getIdPublicacion());
		publicacion.setListaComentarios(comentarios);
	}
	
	private void _completarComentario (Comentario comentario)
	{
		Usuario usuario = retrieveUser(comentario.getIdUsuario());
		comentario.setUsuario(usuario);
	}
	
	private List _completarSeguimientos (List result)
	{
		List retList = new ArrayList();
		
		Iterator it = result.iterator();
		while (it.hasNext()) {
			Usuario usuario = retrieveUser((Integer)it.next());
			retList.add(usuario);
		}
		return retList;
	}
}