package database;

import java.sql.Blob;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;

import database.utility.CFiltro;
import database.utility.DaoException;
import database.utility.DatabaseDaoDb;
import entity.hibernate.Publicacion;

/*
 * **************************************************************************
 * <p>PublicacionDaoDb dao class extends DatabaseDaoDb
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */
 
public class PublicacionDaoDb extends DatabaseDaoDb
{
	private java.sql.Connection _connection;
	private String DATABASE_SELECT = "SELECT * FROM PUBLICACIONES"; 
	private String DATABASE_INSERT = "INSERT INTO PUBLICACIONES (IdUsuario, IdPublicacion, FechaPublicacion, Texto, Imagen, Link, Zona, Barrio, Direccion) VALUES (?,?,?,?,?,?,?,?,?)"; 
	  
	public static final String DB_IDUSUARIO 		= "IdUsuario";
	public static final String DB_IDPUBLICACION 	= "IdPublicacion";
	public static final String DB_FECHAPUBLICACION 	= "FechaPublicacion";
	public static final String DB_TEXTO 			= "Texto";
	public static final String DB_IMAGEN 			= "Imagen";	
	public static final String DB_LINK 				= "Link";	
	public static final String DB_ZONA 				= "Zona";	
	public static final String DB_BARRIO 			= "Barrio";	
	public static final String DB_DIRECCION 		= "Direccion";	

	public PublicacionDaoDb (java.sql.Connection connection) 
	{
		_connection = connection;      
	}
	
	public Vector consultar(CFiltro filtro, Vector orden) throws DaoException {
		String sentencia = DATABASE_SELECT;
		sentencia += DatabaseDaoDb.construirWhere (filtro);
		sentencia += DatabaseDaoDb.construirOrderBy (orden);		
		ResultSet srs = DatabaseDaoDb.createStatement(_connection, sentencia);
		return _obtenerDatosResultSet (srs);
	}
	
	public Vector consultar(CFiltro filtro) throws DaoException {
		String sentencia = DATABASE_SELECT;
		sentencia += DatabaseDaoDb.construirWhere (filtro);
		ResultSet srs = DatabaseDaoDb.createStatement(_connection, sentencia);
		return _obtenerDatosResultSet (srs);
	}
	
	public Vector consultar() throws DaoException {
		String sentencia = DATABASE_SELECT;
		ResultSet srs = DatabaseDaoDb.createStatement(_connection, sentencia);
		return _obtenerDatosResultSet (srs);
	}
	
	private Vector _obtenerDatosResultSet (ResultSet srs) throws DaoException {
		try {
			Vector retVector = new Vector();
			while (srs.next())	{
				int idUsuario = srs.getInt(DB_IDUSUARIO);
				int idPublicacion = srs.getInt(DB_IDPUBLICACION);
				Date fechaPublicacion = srs.getTimestamp(DB_FECHAPUBLICACION);
				String texto = srs.getString(DB_TEXTO);
				Blob imagen = srs.getBlob(DB_IMAGEN);
				String link = srs.getString(DB_LINK);
				int idZona = srs.getInt(DB_ZONA);
				int idBarrio = srs.getInt(DB_BARRIO);
				String direccion = srs.getString(DB_DIRECCION);
				Publicacion publicacion = new Publicacion (idUsuario, idPublicacion, fechaPublicacion, texto, imagen, link, idZona, idBarrio, direccion); 
				retVector.addElement(publicacion);
			}
			return retVector;
		  
		} catch(SQLException ex) {
			throw new DaoException(DaoException.EXC_SQLException, ex.getMessage());
		}
	}
	
	public boolean insert(Publicacion publicacion) throws DaoException
	{
		boolean retCode = false;
		SimpleDateFormat dtf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");

		PreparedStatement stmt;
		try {
			stmt = _connection.prepareStatement(DATABASE_INSERT);

			stmt.setInt(1, publicacion.getIdUsuario());
			stmt.setNull(2, java.sql.Types.INTEGER);
			stmt.setString(3, dtf.format(publicacion.getFechaPublicacion()));
			if (publicacion.getTexto()!=null && !publicacion.getTexto().equals("")) stmt.setString(4, publicacion.getTexto());
			else stmt.setNull(4, java.sql.Types.VARCHAR);
			if (publicacion.getImagen()!=null) stmt.setBinaryStream(5, publicacion.getImagen().getBinaryStream(), (int) publicacion.getImagen().length());
			else stmt.setNull(5, java.sql.Types.BLOB);
			if (publicacion.getLink()!=null && !publicacion.getLink().equals("")) stmt.setString(6, publicacion.getLink());
			else stmt.setNull(6, java.sql.Types.VARCHAR);
			if (publicacion.getIdZona()!=null) stmt.setInt(7, publicacion.getIdZona());
			else stmt.setNull(7, java.sql.Types.INTEGER);
			if (publicacion.getIdBarrio()!=null) stmt.setInt(8, publicacion.getIdBarrio());
			else stmt.setNull(8, java.sql.Types.INTEGER);
			if (publicacion.getDireccion()!=null && !publicacion.getDireccion().equals("")) stmt.setString(9, publicacion.getDireccion());
			else stmt.setNull(9, java.sql.Types.VARCHAR);
			
			stmt.execute();
			retCode = true;
			
		} catch(SQLException ex) {
			throw new DaoException(DaoException.EXC_SQLException, ex.getMessage());
		}		
		
		return retCode;
	}
	
	public boolean delete(Publicacion entity) throws DaoException
	{
		return false;
	}
	
	public boolean update(Publicacion entity) throws DaoException
	{
		return false;
	}
	
}