package database;
 
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;

import database.utility.CFiltro;
import database.utility.DaoException;
import database.utility.DatabaseDaoDb;
import entity.hibernate.Usuario;
 
/*
 * **************************************************************************
 * <p>UsuarioDaoDb dao class extends HibernateDaoDb
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */
 
public class UsuarioDaoDb extends DatabaseDaoDb
{
	private java.sql.Connection _connection;
	private String DATABASE_SELECT = "SELECT * FROM USUARIOS"; 
	private String DATABASE_UPDATE = "UPDATE USUARIOS";
	private String DATABASE_INSERT = "INSERT INTO USUARIOS (IdUsuario, Password, TipoUsuario, Nombre, Apellidos, FechaNacimiento, Direccion, CodigoPostal, Pais, Email) VALUES (?,?,?,?,?,?,?,?,?,?)"; 

	public static final String DB_IDUSUARIO 		= "IdUsuario";
	public static final String DB_PASSWORD	 		= "Password";
	public static final String DB_TIPOUSUARIO		= "TipoUsuario";
	public static final String DB_NOMBRE			= "Nombre";
	public static final String DB_APELLIDOS 		= "Apellidos";
	public static final String DB_FECHANACIMIENTO	= "FechaNacimiento";
	public static final String DB_DIRECCION			= "Direccion";
	public static final String DB_CODIGOPOSTAL		= "CodigoPostal";
	public static final String DB_PAIS 				= "Pais";
	public static final String DB_EMAIL				= "Email";
	

	public UsuarioDaoDb (java.sql.Connection connection)
	{
		_connection = connection;   
	}
	
	public Vector consultar(CFiltro filtro, Vector orden) throws DaoException {
		String sentencia = DATABASE_SELECT;
		sentencia += DatabaseDaoDb.construirWhere (filtro);
		sentencia += DatabaseDaoDb.construirOrderBy (orden);		
		ResultSet srs = DatabaseDaoDb.createStatement(_connection, sentencia);
		return _obtenerDatosResultSet (srs);
	}
	
	public Vector consultar(CFiltro filtro) throws DaoException {
		String sentencia = DATABASE_SELECT;
		sentencia += DatabaseDaoDb.construirWhere (filtro);
		ResultSet srs = DatabaseDaoDb.createStatement(_connection, sentencia);
		return _obtenerDatosResultSet (srs);
	}
	
	public Vector consultar() throws DaoException {
		String sentencia = DATABASE_SELECT;
		ResultSet srs = DatabaseDaoDb.createStatement(_connection, sentencia);
		return _obtenerDatosResultSet (srs);
	}
	
	private Vector _obtenerDatosResultSet (ResultSet srs) throws DaoException {
		try {
			Vector retVector = new Vector();
			while (srs.next())	{
				int idUsuario = srs.getInt(DB_IDUSUARIO);
				String password = srs.getString(DB_PASSWORD);
				String tipoUsuario = srs.getString(DB_TIPOUSUARIO);
				String nombre = srs.getString(DB_NOMBRE);				
				String apellidos = srs.getString(DB_APELLIDOS);	
				Date fechaNacimiento = srs.getDate(DB_FECHANACIMIENTO);
				String direccion = srs.getString(DB_DIRECCION);
				String codigoPostal = srs.getString(DB_CODIGOPOSTAL);
				String pais = srs.getString(DB_PAIS);
				String email = srs.getString(DB_EMAIL);
				Usuario usuario = new Usuario (idUsuario, password, tipoUsuario, nombre, apellidos, fechaNacimiento, direccion, codigoPostal, pais, email); 
				retVector.addElement(usuario);
			}
			return retVector;
		  
		} catch(SQLException ex) {
			throw new DaoException(DaoException.EXC_SQLException, ex.getMessage());
		}
	}

	public boolean insert(Usuario usuario) throws DaoException
	{
		boolean retCode = false;
		SimpleDateFormat dtf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");

		PreparedStatement stmt;
		try {
			stmt = _connection.prepareStatement(DATABASE_INSERT);

			stmt.setNull(1, java.sql.Types.INTEGER);
			stmt.setString(2, usuario.getPasword());
			stmt.setString(3, usuario.getTipoUsuario());
			if (usuario.getNombre()!=null && !usuario.getNombre().equals("")) stmt.setString(4, usuario.getNombre());
			else stmt.setNull(4, java.sql.Types.VARCHAR);
			if (usuario.getApellidos()!=null && !usuario.getApellidos().equals("")) stmt.setString(5, usuario.getApellidos());
			else stmt.setNull(5, java.sql.Types.VARCHAR);
			if (usuario.getFechaNacimiento()!=null) stmt.setString(6, dtf.format(usuario.getFechaNacimiento()));
			else stmt.setNull(6, java.sql.Types.DATE);
			if (usuario.getDireccion()!=null && !usuario.getDireccion().equals("")) stmt.setString(7, usuario.getDireccion());
			else stmt.setNull(7, java.sql.Types.VARCHAR);
			if (usuario.getCodigoPostal()!=null && !usuario.getCodigoPostal().equals("")) stmt.setString(8, usuario.getCodigoPostal());
			else stmt.setNull(8, java.sql.Types.VARCHAR);
			if (usuario.getPais()!=null && !usuario.getPais().equals("")) stmt.setString(9, usuario.getPais());
			else stmt.setNull(9, java.sql.Types.VARCHAR);
			stmt.setString(10, usuario.getEmail());			
			
			stmt.execute();
			retCode = true;
				
		} catch(SQLException ex) {
			throw new DaoException(DaoException.EXC_SQLException, ex.getMessage());
		}
		
		return retCode;
	}
	
	public boolean delete(Usuario entity) throws DaoException
	{
		return false;	
	}
	
	public boolean update(Usuario entity) throws DaoException
	{
		return false;	
	}
	
}