package database.hibernate;
 
import java.io.Serializable;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import database.utility.DaoException;
 
/*
 * **************************************************************************
 *  <p>HibernateDaoDb dao class
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */
 
public class HibernateDaoDb
{
	protected Session _session = null;
	
	public HibernateDaoDb (Session session)
	{
		_session = session;
	}
	
	public List retrieve(Criteria filter) throws DaoException
	{
		List list = null;
		 
		try
		{
			list=filter.list();
			return list;
		}
		catch(HibernateException exc)
		{
			throw new DaoException(exc, getClass().getName() + " internal error RETRIEVING entity. REASON: "+exc.getMessage());
		}
		
	}	
	
	public boolean insert(Serializable entity) throws DaoException
	{
		boolean retCode = false;
		 
		try
		{
			_session.save(entity);
			retCode = true;
		}
		catch(HibernateException exc)
		{
			throw new DaoException(exc, getClass().getName() + " internal error inserting entity. REASON: "+exc.getMessage());
		}
		
		return retCode;
	}
	
	public boolean delete(Serializable entity) throws DaoException
	{
		boolean retCode = false;
	 
		try
		{
			_session.delete(entity);
			retCode = true;
		}
		catch(HibernateException exc)
		{
			throw new DaoException(exc, getClass().getName() + " internal error deleting entity. REASON: "+exc.getMessage());
		}
	
		return retCode;
	}
	
	public boolean update(Serializable entity) throws DaoException
	{
		boolean retCode = false;
	 
		try
		{
			_session.saveOrUpdate(entity);
			retCode = true;
		}
		catch(HibernateException exc)
		{
			throw new DaoException(exc, getClass().getName() + " internal error updating entity. REASON: "+exc.getMessage());
		}
	
		return retCode;
	}
}