package database.hibernate;
 
import java.io.Serializable;
import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.util.Hashtable;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Expression;


import control.utility.ApplicationGlobals;
import control.utility.Logger;
import database.utility.FabricaHibernateDaoDatabase;
import database.utility.HibernateUtil;
import entity.hibernate.Barrio;
import entity.hibernate.Comentario;
import entity.hibernate.Publicacion;
import entity.hibernate.Usuario;
import entity.hibernate.Zona;


/*
 * **************************************************************************
 * <p>SystemHibernateDaoDbUtility utility class
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */
 
public class SystemHibernateDaoDbUtility 
{
	public SystemHibernateDaoDbUtility()
	{
	}
	
	public static Usuario retrieveUser(String email) throws HibernateException 
	{
		Usuario user = null;
		
		try {
			Session session = HibernateUtil.currentSession(ApplicationGlobals.HIBERNATE_FACTORY);
			FabricaHibernateDaoDatabase daoUtility = new FabricaHibernateDaoDatabase(session);

			Criteria criteria = session.createCriteria(Usuario.class);
			criteria.add(Expression.eq("Email",email));
			
			List list = daoUtility.retrieve(criteria);
			if (list!=null && list.size()>0){
				user = (Usuario)list.get(0);
			}
			//close database transaction...
			HibernateUtil.closeSession(ApplicationGlobals.HIBERNATE_FACTORY);
			
		}
		catch(HibernateException e){
			Logger.log(e.getMessage());
			e.printStackTrace();
		}
		 
		return user;
	}
	
	public static List retrievePublicaciones(Integer idUsuario) throws HibernateException 
	{
		List result = null;
		
		try {
			Session session = HibernateUtil.currentSession(ApplicationGlobals.HIBERNATE_FACTORY);
			FabricaHibernateDaoDatabase daoUtility = new FabricaHibernateDaoDatabase(session);

			Criteria criteria = session.createCriteria(Publicacion.class);
			criteria.add(Expression.eq("IdUsuario", idUsuario));
			result = daoUtility.retrieve(criteria);
			
			//close database transaction...
			HibernateUtil.closeSession(ApplicationGlobals.HIBERNATE_FACTORY);
			
		} catch (HibernateException e) {
			Logger.log(e.getMessage());
			e.printStackTrace();
		}
		
		return result;
	}

	public static List retrieveSeguimientos(Integer idUsuario) throws HibernateException 
	{
		List result = null;
		try {
			Session session = HibernateUtil.currentSession(ApplicationGlobals.HIBERNATE_FACTORY);
			FabricaHibernateDaoDatabase daoUtility = new FabricaHibernateDaoDatabase(session);	
			Criteria criteria = session.createCriteria(Usuario.class);
			criteria.add(Expression.eq("IdUsuario", idUsuario));
			result = daoUtility.retrieve(criteria);
			HibernateUtil.closeSession(ApplicationGlobals.HIBERNATE_FACTORY);
			
		} catch (HibernateException e) {
			Logger.log(e.getMessage());
			e.printStackTrace();
		}
		
		return result;
	}
	
	public static List retrievePublicaciones(Integer idZona, Integer idBarrio, Integer idUsuario) throws HibernateException 
	{
		List result = null;
		try {
			Session session = HibernateUtil.currentSession(ApplicationGlobals.HIBERNATE_FACTORY);
			FabricaHibernateDaoDatabase daoUtility = new FabricaHibernateDaoDatabase(session);	
			Criteria criteria = session.createCriteria(Publicacion.class);
			if (idZona != 0) criteria.add(Expression.eq("IdZona", idZona));
			if (idBarrio != 0) criteria.add(Expression.eq("IdBarrio", idBarrio));
			if (idUsuario != 0) criteria.add(Expression.eq("IdUsuario", idUsuario));
			result = daoUtility.retrieve(criteria);
			HibernateUtil.closeSession(ApplicationGlobals.HIBERNATE_FACTORY);
			
		} catch (HibernateException e) {
			Logger.log(e.getMessage());
			e.printStackTrace();
		}
		
		return result;
	}
	
	public static List retrieveUsuarios() throws HibernateException 
	{
		List result = null;
		try {
			Session session = HibernateUtil.currentSession(ApplicationGlobals.HIBERNATE_FACTORY);
			FabricaHibernateDaoDatabase daoUtility = new FabricaHibernateDaoDatabase(session);	
			Criteria criteria = session.createCriteria(Usuario.class);
			result = daoUtility.retrieve(criteria);
			HibernateUtil.closeSession(ApplicationGlobals.HIBERNATE_FACTORY);
			
		} catch (HibernateException e) {
			Logger.log(e.getMessage());
			e.printStackTrace();
		}
		
		return result;
	}
	
	public static List retrieveZonas() throws HibernateException 
	{
		List result = null;
		try {
			Session session = HibernateUtil.currentSession(ApplicationGlobals.HIBERNATE_FACTORY);
			FabricaHibernateDaoDatabase daoUtility = new FabricaHibernateDaoDatabase(session);	
			Criteria criteria = session.createCriteria(Zona.class);
			result = daoUtility.retrieve(criteria);
			HibernateUtil.closeSession(ApplicationGlobals.HIBERNATE_FACTORY);
			
		} catch (HibernateException e) {
			Logger.log(e.getMessage());
			e.printStackTrace();
		}
		
		return result;
	}

	public static List retrieveBarrios() throws HibernateException 
	{
		List result = null;
		try {
			Session session = HibernateUtil.currentSession(ApplicationGlobals.HIBERNATE_FACTORY);
			FabricaHibernateDaoDatabase daoUtility = new FabricaHibernateDaoDatabase(session);	
			Criteria criteria = session.createCriteria(Barrio.class);
			result = daoUtility.retrieve(criteria);
			HibernateUtil.closeSession(ApplicationGlobals.HIBERNATE_FACTORY);
			
		} catch (HibernateException e) {
			Logger.log(e.getMessage());
			e.printStackTrace();
		}
		
		return result;
	}

	public static Comentario retrieveComment(Integer idComentario) throws HibernateException 
	{
		Comentario comment = null;

		try {
			Session session = HibernateUtil.currentSession(ApplicationGlobals.HIBERNATE_FACTORY);
			FabricaHibernateDaoDatabase daoUtility = new FabricaHibernateDaoDatabase(session);	
			
			Criteria criteria = session.createCriteria(Comentario.class);
			criteria.add(Expression.eq("IdComentario", idComentario));
		
			List list = daoUtility.retrieve(criteria);
			if (list!=null && list.size()>0){
				comment = (Comentario)list.get(0);
			}
			
			HibernateUtil.closeSession(ApplicationGlobals.HIBERNATE_FACTORY);
			
		} catch (HibernateException e) {
			Logger.log(e.getMessage());
			e.printStackTrace();
		}

		return comment;
	}
	
	public static List retrieveComentariosPublicacion(String idUsuario, String idPublicacion) throws HibernateException 
	{
		List result = null;
		
		try {
			Session session = HibernateUtil.currentSession(ApplicationGlobals.HIBERNATE_FACTORY);
			FabricaHibernateDaoDatabase daoUtility = new FabricaHibernateDaoDatabase(session);	
			Criteria criteria = session.createCriteria(Comentario.class);
			result = daoUtility.retrieve(criteria);
			criteria.add(Expression.eq("IdUsuario", idUsuario));
			criteria.add(Expression.eq("IdPublicacion", idPublicacion));
			HibernateUtil.closeSession(ApplicationGlobals.HIBERNATE_FACTORY);
			
		} catch (HibernateException e) {
			Logger.log(e.getMessage());
			e.printStackTrace();
		}
		
		return result;
	}
	
	public static boolean insertUsuario(Usuario entity)
	{
		boolean retCode = false;
		
		try
		{
			Session session = HibernateUtil.currentSession(ApplicationGlobals.HIBERNATE_FACTORY);
			FabricaHibernateDaoDatabase daoUtility = new FabricaHibernateDaoDatabase(session);
			Transaction transaction = session.beginTransaction();
			
			if (daoUtility.insert(entity)) {
				transaction.commit();
				retCode = true;
			}
			else
			{
				transaction.rollback();
			}
			
			HibernateUtil.closeSession(ApplicationGlobals.HIBERNATE_FACTORY);
		}
		catch(HibernateException e)
		{
			Logger.log(e.getMessage());
			e.printStackTrace();	
		}
		
		return retCode;
	}
	
	public static boolean updateUsuario(Usuario entity)
	{
		boolean retCode = false;
		
		try
		{
			Session session = HibernateUtil.currentSession(ApplicationGlobals.HIBERNATE_FACTORY);
			FabricaHibernateDaoDatabase daoUtility = new FabricaHibernateDaoDatabase(session);
			Transaction transaction = session.beginTransaction();
			
			if (daoUtility.update(entity)) {
				transaction.commit();
				retCode = true;
			}
			else
			{
				transaction.rollback();
			}
			
			HibernateUtil.closeSession(ApplicationGlobals.HIBERNATE_FACTORY);
		}
		catch(HibernateException e)
		{
			Logger.log(e.getMessage());
			e.printStackTrace();	
		}
		
		return retCode;
	}
	
	public static boolean insertPublicacion(Publicacion entity)
	{
		boolean retCode = false;
		
		try
		{
			Session session = HibernateUtil.currentSession(ApplicationGlobals.HIBERNATE_FACTORY);
			FabricaHibernateDaoDatabase daoUtility = new FabricaHibernateDaoDatabase(session);
			Transaction transaction = session.beginTransaction();
			
			if (daoUtility.insert(entity)) {
				transaction.commit();
				retCode = true;
			}
			else
			{
				transaction.rollback();
			}
			
			HibernateUtil.closeSession(ApplicationGlobals.HIBERNATE_FACTORY);
		}
		catch(HibernateException e)
		{
			Logger.log(e.getMessage());
			e.printStackTrace();	
		}
		
		return retCode;
	}
	
	public static boolean insertComentario(Comentario entity)
	{
		boolean retCode = false;
		
		//start database transaction...
		try
			{
				Session session = HibernateUtil.currentSession(ApplicationGlobals.HIBERNATE_FACTORY);
				FabricaHibernateDaoDatabase daoUtility = new FabricaHibernateDaoDatabase(session);
				Transaction transaction = session.beginTransaction();
				
				if (daoUtility.insert(entity)) {
					transaction.commit();
					retCode = true;
				}
				else
				{
					transaction.rollback();
				}
				
				HibernateUtil.closeSession(ApplicationGlobals.HIBERNATE_FACTORY);
			}
			catch (HibernateException e){
				Logger.log(e.getMessage());
				e.printStackTrace();			
	 	 	}
		
		return retCode;
	}
	
	public static boolean insertSeguimientos(Usuario user, List bean)
	{
		boolean retCode = false;
		
		//start database transaction...

		try
		{
			Session session = HibernateUtil.currentSession(ApplicationGlobals.HIBERNATE_FACTORY);
			FabricaHibernateDaoDatabase daoUtility = new FabricaHibernateDaoDatabase(session);
			Transaction transaction = session.beginTransaction();
						
			if (bean == null){
				user.getListaSeguimientos().addAll(bean);
				daoUtility.update(user);
				transaction.commit();
				retCode = true;
			}
			else
			{
				transaction.rollback();
			}
			
			HibernateUtil.closeSession(ApplicationGlobals.HIBERNATE_FACTORY);
		}
		catch (HibernateException e){
			Logger.log(e.getMessage());
			e.printStackTrace();			
 	 	}
		
		return retCode;
	}
}