package database.utility;
 
import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.Vector;
import java.sql.*;

 
/*
 * **************************************************************************
 *  <p>DatabaseDaoDb dao class
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */
 
public class DatabaseDaoDb
{
	private static Statement _stmt = null;
	
	public DatabaseDaoDb ()
	{
	}
	
	public static ResultSet createStatement(Connection connection, String sentencia) throws DaoException {
		try {
			Statement stmt = connection.createStatement(
								   ResultSet.TYPE_SCROLL_INSENSITIVE, 
								   ResultSet.CONCUR_READ_ONLY);
			ResultSet srs = stmt.executeQuery(sentencia);
			return srs;
		} catch(SQLException ex) {
			if (ex.getErrorCode()==2292){
				throw new DaoException(DaoException.EXC_ViolacionRI, ex.getMessage());
			}
			if (ex.getErrorCode()==2291){
				throw new DaoException(DaoException.EXC_ViolacionRI, ex.getMessage());
			}
			else if (ex.getErrorCode()==1){
				throw new DaoException(DaoException.EXC_RegistroDuplicado, ex.getMessage());
			}
			if (ex.getErrorCode()==2290){
				throw new DaoException(DaoException.EXC_ConstraintViolated, ex.getMessage());
			}
			throw new DaoException(ex.getMessage());
		}
	}
	
	public static String construirWhere (CFiltro filtro)
	{
		if (filtro == null) return "";
		
		String where = " WHERE ";
		if (filtro.getBooleano() != CFiltro.NADA) where += "(";
		where += _construirCondicion(filtro.getCondicion());
		int aux = 0;		
		while (filtro.getBooleano() != CFiltro.NADA) {
			if (filtro.getBooleano() == CFiltro.AND) where += " AND ";
			if (filtro.getBooleano() == CFiltro.OR) where += " OR ";
			filtro = filtro.getSiguiente();
			if (filtro.getBooleano() == CFiltro.VACIO) {
				where += _construirCondicion(filtro.getCondicion());
				continue;
				}			
			if (filtro.getBooleano() != CFiltro.NADA) where += "(";	
			where += _construirCondicion(filtro.getCondicion());
			aux++;		
		}
		for (int i = 0; i < aux; i++) where += ")"; 
	
		return where;
	}
	
	public static String construirWhere (String filtro)
	{
		if (filtro == null) return "";
		
		String where = " WHERE "+filtro;
		return where;
	}

	private static String _construirCondicion (CFiltroCondicion condicion) 
	{
		if (condicion==null) return "";
		if (condicion.getValor() == null && 
			condicion.getComparador() == CFiltroCondicion.COMPARADOR_IGUAL)
			return condicion.getCampo() + " IS NULL"; 
		else if (condicion.getValor() == null && 
				 condicion.getComparador() == CFiltroCondicion.COMPARADOR_DIFERENTE)
			return condicion.getCampo() + " IS NOT NULL"; 
		else if (CFiltroCondicion.OPERADOR_TO_DATE.equals(condicion.getOperadorValor())){
			SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
			//String idioma = (condicion.getIdioma()!=null)?condicion.getIdioma():"";
			return _construirOperador(condicion.getOperadorCampo(), condicion.getCampo()) 
				   + condicion.getComparador() 
				   + "to_date('"+df.format(condicion.getValor())+"','YYYY-MM-DD')";
		}
		else	
			return _construirOperador(condicion.getOperadorCampo(), condicion.getCampo()) 
				   + condicion.getComparador() 
				   + _construirOperador(condicion.getOperadorValor(), "'" + condicion.getValor() + "'");
	}
	
	private static String _construirOperador (String operador, String valor) 
	{
		String where = "";
		if (operador != null) where += operador + "(";
		where += valor;
		if (operador != null) where += ")";
		return where;
	}
	
	
	public static String construirOrderBy (Vector orden)
	{
		if (orden == null || orden.size() == 0) return "";
		
		String orderBy = " ORDER BY " + orden.elementAt(0);
	
		for (int i = 1; i < orden.size(); i++)
			orderBy += ", " + orden.elementAt(i);
		
		return orderBy;
	}
	
	public static ResultSet createStatementUpdatable(Connection connection, String sentencia) throws DaoException {
		try {
			_stmt = connection.createStatement(
								 ResultSet.TYPE_SCROLL_SENSITIVE, 
								 ResultSet.CONCUR_UPDATABLE);
			ResultSet uprs = _stmt.executeQuery(sentencia);
			return uprs;
		} catch(SQLException ex) {
			if (ex.getErrorCode()==2292){
				throw new DaoException(DaoException.EXC_ViolacionRI, ex.getMessage());
			}
			if (ex.getErrorCode()==2291){
				throw new DaoException(DaoException.EXC_ViolacionRI, ex.getMessage());
			}
			else if (ex.getErrorCode()==1){
				throw new DaoException(DaoException.EXC_RegistroDuplicado, ex.getMessage());
			}
			if (ex.getErrorCode()==2290){
				throw new DaoException(DaoException.EXC_ConstraintViolated, ex.getMessage());
			}
			throw new DaoException(ex.getMessage());
		}
	}

	public static PreparedStatement createPreparedStatementUpdatable(Connection connection, String sentencia) throws DaoException {
		try {
			PreparedStatement stmt = connection.prepareStatement(sentencia, 
								 ResultSet.TYPE_SCROLL_SENSITIVE, 
								 ResultSet.CONCUR_UPDATABLE);
			return stmt;
		} catch(SQLException ex) {
			if (ex.getErrorCode()==2292){
				throw new DaoException(DaoException.EXC_ViolacionRI, ex.getMessage());
			}
			if (ex.getErrorCode()==2291){
				throw new DaoException(DaoException.EXC_ViolacionRI, ex.getMessage());
			}
			else if (ex.getErrorCode()==1){
				throw new DaoException(DaoException.EXC_RegistroDuplicado, ex.getMessage());
			}
			if (ex.getErrorCode()==2290){
				throw new DaoException(DaoException.EXC_ConstraintViolated, ex.getMessage());
			}
			throw new DaoException(ex.getMessage());
		}
	}
	
	public static void closeCurrentStatement(){
		if (_stmt!=null){
			try {
				_stmt.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
			_stmt=null;
		}
	}

	
}