package database.utility;

import java.util.Hashtable;

import org.hibernate.HibernateException;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.Session;

import control.utility.ApplicationException;
import control.utility.ApplicationGlobals;

/*
 * **************************************************************************
 * <p>HibernateUtil utility class
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */
 
public class HibernateUtil
{
	private static final Hashtable sessionFactories = new Hashtable();
	
	public static final ThreadLocal session = new ThreadLocal();
	
	static 
	{
		String fQssh = "/WEB-INF/classes/" + ApplicationGlobals.HIBERNATE_FACTORY;
		
		try 
		{
		
		Configuration c = new Configuration().configure(fQssh);

			//Creation of the SessionFactory
			
			SessionFactory sf = c.buildSessionFactory();
				
			if (sf != null)
			{
				sessionFactories.put(ApplicationGlobals.HIBERNATE_FACTORY, sf);
			}
			
		} 
		catch (HibernateException e) 
		{
			throw new RuntimeException("Exception building SessionFactory:" + e.getMessage());
		}	
	}
	
	//Get the current session...
	
	public static Session currentSession(String sessionId) throws HibernateException 
	{
		Hashtable h = (Hashtable) session.get();
		
		//Open a new session if needed
		
		if (h == null)
		{
			h = new Hashtable();
			session.set(h); 
		}
		
		Session s = (Session) h.get(sessionId);
		
		if (s == null) 
		{
			//get session factory...
			
			SessionFactory sessionFactory = (SessionFactory) sessionFactories.get(sessionId);
			
			//create session...
			
			if (sessionFactory != null)
			{
				s = sessionFactory.openSession();
				h.put(sessionId, s); 
			}
		}
		
		return s;
	}
	
	//Opens the session...
	
	public static Session openSession(String sessionId) throws HibernateException 
	{
		Hashtable h = (Hashtable) session.get();
		Session s = null;

		if (h != null)
		{		
			s = (Session) h.get(sessionId);
		}
		
		//Open a new session if no one is opened
		
		if (s == null) 
		{
			//get session factory...
			
			SessionFactory sessionFactory = (SessionFactory) sessionFactories.get(sessionId);
			
			//create session...
			
			if (sessionFactory != null)
			{
				s = sessionFactory.openSession();
			}

		}
		 
		return s;
	}	

	//Closes the session...
	
	public static void closeSession(String sessionId) throws HibernateException 
	{
		Hashtable h = (Hashtable) session.get();
		Session s = null;
		
		if (h != null)
		{
			s = (Session) h.get(sessionId);
			if (s != null) 
			{
				s.close();
			}
			
			h.remove(sessionId);
		
			if (h.size() == 0)
			{
				session.set(null);
			}
		}		
	} 
	
}
