package usecase;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import control.factory.MessageFactory;
import control.interfaces.DataEntryUseCase;
import control.interfaces.UseCase;
import control.utility.ApplicationError;
import control.utility.ApplicationEvent;
import control.utility.ApplicationException;
import control.utility.ApplicationGlobals;
import control.utility.ApplicationStore;
import control.utility.BusinessGlobals;
import control.utility.JSPDispatcher;
import control.utility.Logger;
import control.utility.Utility;
import database.DatabaseDaoDbUtility;
import database.hibernate.SystemHibernateDaoDbUtility;
import database.utility.HibernateUtil;
import entity.Dictionary;
import entity.hibernate.Usuario;

/*
 * **************************************************************************
 * <p>AccesoUsuario use case class
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */

public class AccesoUsuario implements UseCase, DataEntryUseCase
{	
	protected ApplicationStore _appStore; 

	protected Usuario _loggedUser;

	public AccesoUsuario(ApplicationStore appStore, Dictionary dicc) 
	{
		this._appStore = appStore;
	}

	public ApplicationEvent start() throws ApplicationException 
	{
		//Prototype.init();
		
		if (_appStore == null) 
		{
			throw new ApplicationException();
		}
		
		ApplicationEvent appEvent = new ApplicationEvent();
		appEvent.setTarget(JSPDispatcher.getTarget(JSPDispatcher.AccesoUsuario));
		appEvent.setActiveUseCase(this);
		return appEvent;
	}

	public ApplicationEvent accept(ApplicationEvent appEvent, Dictionary dto) throws ApplicationException
	{
		//get entered user & password from dto...
		
		if (dto == null)
		{
			throw new ApplicationException();
		}
		
		String email = dto.getSingleValue(BusinessGlobals.DTO_FIELD_SCR0001_USERNAME);
		String password = dto.getSingleValue(BusinessGlobals.DTO_FIELD_SCR0001_PASSWORD);		
		
		ApplicationEvent retAppEvent = new ApplicationEvent();
		appEvent.setTarget(JSPDispatcher.getTarget(JSPDispatcher.Layout));
		retAppEvent.setActiveUseCase(this);
		
		//retrieve user data from Prototype...
//		_loggedUser = Prototype.retrieveUser(email);
		
		//retrieve user data from persistence hibernate...
//		_loggedUser = SystemHibernateDaoDbUtility.retrieveUser(email);
		
		//retrieve user data from persistence...
		DatabaseDaoDbUtility systemDaoDbUtility = new DatabaseDaoDbUtility();
		_loggedUser = systemDaoDbUtility.retrieveUser(email);
		
		if (_loggedUser == null)
		{
			ApplicationError error [] = new ApplicationError[1];
			error[0] = new ApplicationError(MessageFactory.system_login_error_unknownUser, ApplicationError.INTERNAL_APPLICATION_ERROR);
			retAppEvent.addApplicationErrors(error);
			retAppEvent.setActiveUseCase(this);

			return retAppEvent;
		}
		
		//check whether user & password are valid...
	
		ApplicationError[] errorsValidatingUser = _validateUser(password);
	
		if (errorsValidatingUser != null && errorsValidatingUser.length > 0)
		{
			retAppEvent.addApplicationErrors(errorsValidatingUser);
		}

		if (retAppEvent.getApplicationErrors()==null || retAppEvent.getApplicationErrors().size()==0)
		{
			Utility.setLoggedUser(_appStore, _loggedUser);
			
			Logger.log("User " + _loggedUser.getIdUsuario() + " logged succesfully");			
			
			//show wellcome page...

			Welcome welcome = new Welcome(_appStore,null);
			return welcome.start();
		}
		else 
		{
			Logger.log("User " + _loggedUser.getIdUsuario() + " NOT logged properly...");
			
			for (int i = 0; i < retAppEvent.getApplicationErrors().size(); i++)
			{
				ApplicationError thisAE = (ApplicationError) retAppEvent.getApplicationErrors().get(i);
				Logger.log(thisAE.toString());	
			}
		}
		
		return retAppEvent;
	}

	public ApplicationEvent cancel(ApplicationEvent appEvent, Dictionary dto) throws ApplicationException
	{
		throw new ApplicationException();
	} 

	public Usuario getLoggedUser() {
		return _loggedUser;
	}

	public void setLoggedUser(Usuario user) {
		_loggedUser = user;
	}
	
	private ApplicationError[] _validateUser(String password) throws ApplicationException 
	{
		
		if (password.equals(_loggedUser.getPasword()))
		{
			return null;
		}
		else
		{
			ApplicationError [] errors = new ApplicationError[1];
			errors[0] = new ApplicationError(MessageFactory.system_login_error_wrongPassword, ApplicationError.USER_PASSWORD_INCORRECT);
			return errors;
		}
	}
	
}

