package usecase;

import org.hibernate.HibernateException;
import org.hibernate.Transaction;
import org.hibernate.Session;

import control.factory.MessageFactory;
import control.interfaces.UseCase;
import control.utility.ApplicationError;
import control.utility.ApplicationEvent;
import control.utility.ApplicationException;
import control.utility.ApplicationGlobals;
import control.utility.ApplicationStore;
import control.utility.BusinessGlobals;
import control.utility.Logger;
import database.hibernate.SystemHibernateDaoDbUtility;
import database.utility.FabricaHibernateDaoDatabase;
import database.utility.HibernateUtil;
import entity.Dictionary;
import entity.hibernate.Publicacion;

/*
 * **************************************************************************
 * <p>BorrarPublicacion use case class
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */

public class BorrarPublicacion implements UseCase
{
	private ApplicationStore _as = null;

	private Dictionary _deletedPublication = null;
	
	public BorrarPublicacion(ApplicationStore as, Dictionary deletedPublication)
	{
		_as = as;
		_deletedPublication = deletedPublication;
	}
	
	public ApplicationEvent start() throws ApplicationException
	{
		ApplicationEvent retAE = _as.getEvent();
		boolean retCode = true;
		
		try {
			//start database transaction...
			
			Session session = HibernateUtil.currentSession(ApplicationGlobals.HIBERNATE_FACTORY);
			FabricaHibernateDaoDatabase daoUtility = new FabricaHibernateDaoDatabase(session);
			Transaction transaction = session.beginTransaction();
		 
			//delete given users...
		
			if (_deletedPublication != null)
			{
				String idUsuario = _deletedPublication.getSingleValue(BusinessGlobals.DTO_FIELD_USER_ID);
				String idPublicacion = _deletedPublication.getSingleValue(BusinessGlobals.DTO_FIELD_PUBLICATION_ID);
				
				Publicacion bean = new Publicacion(new Integer(idPublicacion));
				retCode = daoUtility.delete(bean);
			}
			else retCode=false;
		
			//return to maintenance window...
		
			if (!retCode)
			{
				transaction.rollback();
				retAE = _as.getEvent();
				ApplicationError error [] = new ApplicationError[1];
				error[0] = new ApplicationError(MessageFactory.system_edituser_error_persistence, ApplicationError.INTERNAL_APPLICATION_ERROR);
				retAE.addApplicationErrors(error);
			}
			else
			{
				transaction.commit();
				DisplayMainLayout useCase = new DisplayMainLayout(_as, new Dictionary());
				retAE = useCase.start(); 
			}
			HibernateUtil.closeSession(ApplicationGlobals.HIBERNATE_FACTORY);	
		}
		catch (HibernateException exc){
			Logger.log(exc.getMessage());
			exc.printStackTrace();
		}

		return retAE;
	}
}
