package usecase;
 
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import control.interfaces.DataEntryUseCase;
import control.interfaces.UseCase;
import control.utility.ApplicationEvent;
import control.utility.ApplicationException;
import control.utility.ApplicationGlobals;
import control.utility.ApplicationStore;
import control.utility.BusinessGlobals;
import control.utility.DateComparator;
import control.utility.JSPDispatcher;
import control.utility.Logger;
import control.utility.Utility;
import database.DatabaseDaoDbUtility;
import database.hibernate.SystemHibernateDaoDbUtility;
import database.utility.HibernateUtil;
import entity.Dictionary;
import entity.Envelope;
import entity.hibernate.Publicacion;
import entity.hibernate.Usuario;

/*
 * **************************************************************************
 * <p>BuscarPublicaciones use case class
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */
 
public class BuscarPublicaciones implements UseCase, DataEntryUseCase 
{	
	protected ApplicationStore _appStore; 
	
	public BuscarPublicaciones(ApplicationStore appStore)
	{
		_appStore = appStore;
	}

	public BuscarPublicaciones(ApplicationStore appStore, Dictionary dicc) 
	{
		this._appStore = appStore;
	}

	public ApplicationEvent start() throws ApplicationException 
	{
		ApplicationEvent retAE = new ApplicationEvent();
		 
		Object [] output = new Object[4];

		//i am adding a new seguimiento usuario into the system...
			
		Usuario user = Utility.getLoggedUser(_appStore);
		output[0] = user;
		
		//retrieve zonas data from Prototype...
//		List zonas = Prototype.retrieveZonas();
		
		//retrieve zonas data from persistence hibernate...
//		List zonas = SystemHibernateDaoDbUtility.retrieveZonas();
		
		//retrieve zonas data from persistence...
		DatabaseDaoDbUtility systemDaoDbUtility = new DatabaseDaoDbUtility();
		List zonas = systemDaoDbUtility.retrieveZonas();
		output[1] = zonas;
		
		//retrieve barrios data from Prototype...
//		List barrios = Prototype.retrieveBarrios();
		
		//retrieve barrios data from persistence hibernate...
//		List barrios = SystemHibernateDaoDbUtility.retrieveBarrios();
		
		//retrieve barrios data from persistence...
		List barrios = systemDaoDbUtility.retrieveBarrios();
		output[2] = barrios;
		
		//retrieve usuarios data from Prototype...
//		List usuarios = Prototype.retrieveUsuarios();
		
		//retrieve usuarios data from persistence hibernate...
//		List usuarios = SystemHibernateDaoDbUtility.retrieveUsuarios();
		
		//retrieve usuarios data from persistence...
		List usuarios = systemDaoDbUtility.retrieveUsuarios();
		output[3] = usuarios;
		
		//build dto & store it in application event...
		//create envelope to jsp & redispatch...
		//envelope contains array with: [0]=publication, [1]=comments...
		//set default window & layout...
		
		Envelope envelope = new Envelope(output);		
				
		retAE.setTarget(JSPDispatcher.getTarget(JSPDispatcher.BuscarPublicacionesCriterios));	
		retAE.setDataModel(envelope);		
		retAE.setActiveUseCase(this);		
			
		return retAE;				

	}

	public ApplicationEvent accept(ApplicationEvent appEvent, Dictionary dto) throws ApplicationException 
	{
		ApplicationEvent retAE = new ApplicationEvent();
		Object [] output = new Object[2];
				
		Usuario user = Utility.getLoggedUser(_appStore);
		
		if (user != null)
		{
			output[0] = user;
			
			String idZona = dto.getSingleValue(BusinessGlobals.DTO_FIELD_BUSQUEDA_ZONA);
			String idBarrio = dto.getSingleValue(BusinessGlobals.DTO_FIELD_BUSQUEDA_BARRIO);
			String idUsuario = dto.getSingleValue(BusinessGlobals.DTO_FIELD_BUSQUEDA_USUARIO);

			//retrieve publicaciones data from Prototype...
//			List publicaciones = Prototype.retrievePublicaciones(idZona!=null ? new Integer(idZona):0, idBarrio!=null ? new Integer(idBarrio):0, idUsuario!=null ? new Integer(idUsuario):0);
			
			//retrieve publicaciones data from persistence hibernate...
//			List publicaciones = SystemHibernateDaoDbUtility.retrievePublicaciones(idZona, idBarrio, idUsuario());
			
			//retrieve publicaciones data from persistence...
			DatabaseDaoDbUtility systemDaoDbUtility = new DatabaseDaoDbUtility();
			List publicaciones = systemDaoDbUtility.retrievePublicaciones(idZona!=null ? new Integer(idZona):null, idBarrio!=null ? new Integer(idBarrio):null, idUsuario!=null ? new Integer(idUsuario):null); 
					
			DateComparator comparator = new DateComparator(false);
			Collections.sort(publicaciones, comparator);
				
			output[1] = publicaciones;
		}
		
		//build dto & store it in application event...
		//create envelope to jsp & redispatch...
		//envelope contains array with: [0]=usuario, [1]=publication, [2]=seguimientos...
		//set default window & layout...
		
		Envelope envelope = new Envelope(output);		
				
		retAE.setTarget(JSPDispatcher.getTarget(JSPDispatcher.BuscarPublicaciones));	
		retAE.setDataModel(envelope);		
		retAE.setActiveUseCase(new DisplayMainLayout(_appStore, null));		
			
		return retAE;
	}
	
	public ApplicationEvent cancel(ApplicationEvent appEvent) throws ApplicationException 
	{
		DisplayMainLayout useCase = new DisplayMainLayout(_appStore, new Dictionary());
		return useCase.start(); 
	}
	
	public ApplicationEvent cancel(ApplicationEvent appEvent, Dictionary dicc) throws ApplicationException {
		DisplayMainLayout useCase = new DisplayMainLayout(_appStore, new Dictionary());
		return useCase.start(); 
	}
	
	 
}

