package usecase;
 
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import control.factory.MessageFactory;
import control.interfaces.DataEntryUseCase;
import control.interfaces.UseCase;
import control.utility.ApplicationError;
import control.utility.ApplicationEvent;
import control.utility.ApplicationException;
import control.utility.ApplicationGlobals;
import control.utility.ApplicationStore;
import control.utility.BusinessGlobals;
import control.utility.JSPDispatcher;
import control.utility.Logger;
import control.utility.Utility;
import database.DatabaseDaoDbUtility;
import database.hibernate.SystemHibernateDaoDbUtility;
import database.utility.HibernateUtil;
import entity.Dictionary;
import entity.Envelope;
import entity.hibernate.Barrio;
import entity.hibernate.Comentario;
import entity.hibernate.Publicacion;
import entity.hibernate.Usuario;
import entity.hibernate.Zona;

/*
 * **************************************************************************
 * <p>ComentarPublicacion use case class
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */

public class ComentarPublicacion implements UseCase, DataEntryUseCase
{
	private ApplicationStore _as = null;

	private Dictionary _dto = null;
	
	public ComentarPublicacion(ApplicationStore as)
	{
		_as = as;
	}
	
	public ComentarPublicacion(ApplicationStore as, Dictionary dto)
	{
		_as = as;
		_dto = dto;
	}
	
	public ApplicationEvent start() throws ApplicationException
	{
		ApplicationEvent retAE = new ApplicationEvent();
		 
		//recover user data if updating publication...
		
		Usuario user = Utility.getLoggedUser(_as);
		
		Object [] output = new Object[2];
		
		//i am adding a new commnet into publication...
			
		//set default window & layout...
		
		//recover available ...

		String idPublicacion = _dto.getSingleValue(ApplicationGlobals.SYSTEM_PARAM_ID_EVENT_PARAMETERS);
		
		//retrieve publicacion data from Prototype...
//		Publicacion publicacion = Prototype.retrievePublicacion(new Integer(idPublicacion)); 
		
		//retrieve publicacion data from persistence hibernate...
//		Publicacion publicacion = SystemHibernateDaoDbUtility.retrievePublicacion(idPublicacion); 
				
		//retrieve publicacion data from persistence...
		DatabaseDaoDbUtility systemDaoDbUtility = new DatabaseDaoDbUtility();
		Publicacion publicacion = systemDaoDbUtility.retrievePublicacion(new Integer(idPublicacion));
		
		output[0] = publicacion;
		
		Comentario newComment = new Comentario(publicacion.getIdPublicacion(), null);
		newComment.setFechaComentario(new Date());
		newComment.setIdUsuario(user.getIdUsuario());

		output[1] = newComment;
		
		Envelope envelope = new Envelope(output);		
				
		retAE.setTarget(JSPDispatcher.getTarget(JSPDispatcher.ComentarPublicacion));	
		retAE.setDataModel(envelope);		
		retAE.setActiveUseCase(this);		
			
		return retAE;				
	}

	public ApplicationEvent accept(ApplicationEvent appEvent, Dictionary dto) throws ApplicationException 
	{
		ApplicationEvent retAE = new ApplicationEvent();
		boolean consolidated  = false;
		Comentario bean = null; 
		
		//set default window & layout...
		
		retAE.setTarget(JSPDispatcher.getTarget(JSPDispatcher.ComentarPublicacion));
		
	    //build bean from dto...
		bean = _buildComment(dto);
		
		//insert comentario data from persistence hibernate...		 
//		if (SystemHibernateDaoDbUtility.insertComentario(bean)) {
		
		//insert comentario data from persistence...
		DatabaseDaoDbUtility systemDaoDbUtility = new DatabaseDaoDbUtility();
		if (systemDaoDbUtility.insertComentario(bean)) {
			consolidated = true;
		}
		else
		{
			ApplicationError error [] = new ApplicationError[1];
			error[0] = new ApplicationError(MessageFactory.system_edituser_error_persistence, ApplicationError.INTERNAL_APPLICATION_ERROR);
			retAE.addApplicationErrors(error);
		}

		ArrayList errorsAE = retAE.getApplicationErrors();
 		
		if (consolidated)
		{
			DisplayMainLayout usecase = new DisplayMainLayout(_as, new Dictionary());
			retAE = usecase.start();
			
			ApplicationError error [] = new ApplicationError[1];
			error[0] = new ApplicationError(MessageFactory.system_edituser_success, ApplicationError.NO_ERROR);
			retAE.addApplicationErrors(error);
		}
		else
		{
			retAE = start();
			retAE.setApplicationErrors(errorsAE);
		}		
				
		return retAE;		
	}
	
	public ApplicationEvent cancel(ApplicationEvent appEvent, Dictionary dto) throws ApplicationException
	{
		DisplayMainLayout useCase = new DisplayMainLayout(_as, new Dictionary());
		return useCase.start(); 
	}

	public ApplicationEvent cancel(ApplicationEvent appEvent) throws ApplicationException 
	{
		DisplayMainLayout useCase = new DisplayMainLayout(_as, new Dictionary());
		return useCase.start(); 
	}

	private Comentario _buildComment(Dictionary dto)
	{
		Comentario bean = null;
		
		String idPublicacion = dto.getSingleValue(BusinessGlobals.DTO_FIELD_PUBLICATION_ID);
		
		if (idPublicacion != null)
		{
			bean = new Comentario();
			bean.setIdPublicacion(new Integer(idPublicacion));

			bean.setIdUsuario(new Integer(dto.getSingleValue(BusinessGlobals.DTO_FIELD_COMMENT_IDUSUARIOCOMENTARIO)));
			SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
			try {
				bean.setFechaComentario(df.parse(dto.getSingleValue(BusinessGlobals.DTO_FIELD_COMMENT_FECHACOMENTARIO)));
			} catch (ParseException e) {
				e.printStackTrace();
			}
			bean.setTexto(dto.getSingleValue(BusinessGlobals.DTO_FIELD_COMMENT_TEXTO));
		}
		
		return bean;
	}

}
