package usecase;
 
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.sql.Blob;
import java.sql.SQLException;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import control.interfaces.DataEntryUseCase;
import control.interfaces.UseCase;
import control.utility.ApplicationEvent;
import control.utility.ApplicationException;
import control.utility.ApplicationGlobals;
import control.utility.ApplicationStore;
import control.utility.BusinessGlobals;
import control.utility.DateComparator;
import control.utility.JSPDispatcher;
import control.utility.Utility;
import database.DatabaseDaoDbUtility;
import entity.Dictionary;
import entity.Envelope;
import entity.hibernate.Publicacion;
import entity.hibernate.Usuario;

/*
 * **************************************************************************
 * <p>DisplayMainLayout use case class
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */
 
public class DisplayMainLayout implements UseCase, DataEntryUseCase 
{	
	protected ApplicationStore _appStore; 

	public DisplayMainLayout(ApplicationStore appStore, Dictionary dicc) 
	{
		this._appStore = appStore;
	}

	public ApplicationEvent start() throws ApplicationException 
	{
		ApplicationEvent retAE = new ApplicationEvent();
		Object [] output = new Object[3];
				
		retAE.setTarget(JSPDispatcher.getTarget(JSPDispatcher.MainLayout));
		retAE.setActiveUseCase(this);

		Usuario user = Utility.getLoggedUser(_appStore);
		
		if (user != null)
		{
			//retrieve publicaciones data from Prototype...
//			List publicaciones = Prototype.retrievePublicaciones(user.getIdUsuario());
			
			//retrieve user data from persistence hibernate...
//			List publicaciones = SystemHibernateDaoDbUtility.retrievePublicaciones(user.getIdUsuario());

			//retrieve publicaciones data from persistence...
			DatabaseDaoDbUtility systemDaoDbUtility = new DatabaseDaoDbUtility();
			List publicaciones = systemDaoDbUtility.retrievePublicaciones(user.getIdUsuario());

			output[0] = user;
						
			if (user.getTipoUsuario()!=null && BusinessGlobals.PROFILE_EMPRESA.equals(user.getTipoUsuario()))
			{
				output[1] = publicaciones;
			}
			else 
			{
				//retrieve seguimientos data from Prototype...
//				List seguimientos = Prototype.retrieveSeguimientos(user.getIdUsuario());

				//retrieve seguimientos data from persistence hibernate...
//				List seguimientos = SystemHibernateDaoDbUtility.retrieveSeguimientos(user.getIdUsuario());

				//retrieve seguimientos data from persistence...
				List seguimientos = systemDaoDbUtility.retrieveSeguimientos(user.getIdUsuario());
				
				Iterator it= seguimientos.iterator();
				while (it.hasNext()) {
					Integer idUsuario = ((Usuario)it.next()).getIdUsuario();
					
					//retrieve publicaciones data from Prototype...
//					List publicacionesSeguidas = Prototype.retrievePublicaciones(idUsuario);
					
					//retrieve user data from persistence hibernate...
//					List publicacionesSeguidas = SystemHibernateDaoDbUtility.retrievePublicaciones(idUsuario);

					//retrieve publicaciones data from persistence...
					List publicacionesSeguidas = systemDaoDbUtility.retrievePublicaciones(idUsuario);

					publicaciones.addAll(publicacionesSeguidas);
				}
				
				DateComparator comparator = new DateComparator(false);
				Collections.sort(publicaciones, comparator);
				
				output[1] = publicaciones;
				output[2] = seguimientos;
			}
		}
		
		//build dto & store it in application event...
		//create envelope to jsp & redispatch...
		//envelope contains array with: [0]=usuario, [1]=publication, [2]=seguimientos...
		//set default window & layout...
		
		Envelope envelope = new Envelope(output);		
				
		retAE.setTarget(JSPDispatcher.getTarget(JSPDispatcher.MainLayout));	
		retAE.setDataModel(envelope);		
		retAE.setActiveUseCase(this);		
			
		return retAE;
	}

	public ApplicationEvent accept(ApplicationEvent appEvent, Dictionary dicc) throws ApplicationException 
	{
		return null;
	}
	
	public ApplicationEvent cancel(ApplicationEvent appEvent, Dictionary dicc) throws ApplicationException {
		return null;
	}
	 
}

