package usecase;
 
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.Blob;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.sql.rowset.serial.SerialBlob;

import control.factory.MessageFactory;
import control.interfaces.DataEntryUseCase;
import control.interfaces.DataEntryWithFilesUseCase;
import control.interfaces.UseCase;
import control.utility.ApplicationError;
import control.utility.ApplicationEvent;
import control.utility.ApplicationException;
import control.utility.ApplicationStore;
import control.utility.BusinessGlobals;
import control.utility.JSPDispatcher;
import control.utility.Utility;
import database.DatabaseDaoDbUtility;
import entity.Dictionary;
import entity.Envelope;
import entity.hibernate.Publicacion;
import entity.hibernate.Usuario;

/*
 * **************************************************************************
 * <p>PublicarComentario use case class
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */

public class PublicarComentario implements UseCase, DataEntryUseCase, DataEntryWithFilesUseCase
{
	private ApplicationStore _as = null;
	
	public PublicarComentario(ApplicationStore as)
	{
		_as = as;
	}
	
	public PublicarComentario(ApplicationStore as, Dictionary dicc)
	{
		_as = as;
	}
	
	public ApplicationEvent start() throws ApplicationException
	{
		ApplicationEvent retAE = new ApplicationEvent();
		 
		Object [] output = new Object[3];

		//i am adding a new publication into the system...
			
		Usuario user = Utility.getLoggedUser(_as);
		
		Publicacion newPublication = new Publicacion(null);
		newPublication.setIdUsuario(user.getIdUsuario());
		newPublication.setFechaPublicacion(new Date());
		output[0] = newPublication;
		
		//retrieve zonas data from Prototype...
//		List zonas = Prototype.retrieveZonas();
		
		//retrieve zonas data from persistence hibernate...
//		List zonas = SystemHibernateDaoDbUtility.retrieveZonas();
		
		//retrieve zonas data from persistence...
		DatabaseDaoDbUtility systemDaoDbUtility = new DatabaseDaoDbUtility();
		List zonas = systemDaoDbUtility.retrieveZonas();
		output[1] = zonas;
		
		//retrieve barrios data from Prototype...
//		List barrios = Prototype.retrieveBarrios();
		
		//retrieve barrios data from persistence hibernate...
//		List barrios = SystemHibernateDaoDbUtility.retrieveBarrios();
		
		//retrieve barrios data from persistence...
		List barrios = systemDaoDbUtility.retrieveBarrios();
		output[2] = barrios;

		//build dto & store it in application event...
		//create envelope to jsp & redispatch...
		//envelope contains array with: [0]=publication, [1]=comments...
		//set default window & layout...
		
		Envelope envelope = new Envelope(output);		
				
		retAE.setTarget(JSPDispatcher.getTarget(JSPDispatcher.PublicarComentario));	
		retAE.setDataModel(envelope);		
		retAE.setActiveUseCase(this);		
			
		return retAE;				
	}

	public ApplicationEvent accept(ApplicationEvent appEvent, Dictionary dto) throws ApplicationException 
	{
		return null;		
	}
	
	public ApplicationEvent acceptWithFiles(ApplicationEvent appEvent, Dictionary dto, ArrayList files) throws ApplicationException 
	{
		ApplicationEvent retAE = new ApplicationEvent();
		boolean consolidated  = false;
		Publicacion bean = null; 
		
		retAE.setTarget(JSPDispatcher.getTarget(JSPDispatcher.PublicarComentario));
		
	    //build bean from dto...
		bean = _buildPublication(dto, files);
	
		//insert publicacion data from persistence hibernate...		 
//		if (SystemHibernateDaoDbUtility.insertPublicacion(bean)) {
		
		//insert publicacion data from persistence...	
		DatabaseDaoDbUtility systemDaoDbUtility = new DatabaseDaoDbUtility();
		if (systemDaoDbUtility.insertPublicacion(bean)){
				consolidated = true;
		}
		else
		{
			ApplicationError error [] = new ApplicationError[1];
			error[0] = new ApplicationError(MessageFactory.system_edituser_error_persistence, ApplicationError.INTERNAL_APPLICATION_ERROR);
			retAE.addApplicationErrors(error);
		}

		ArrayList errorsAE = retAE.getApplicationErrors();
 		
		if (consolidated)
		{
			DisplayMainLayout usecase = new DisplayMainLayout(_as, new Dictionary());
			retAE = usecase.start();
			
			ApplicationError error [] = new ApplicationError[1];
			error[0] = new ApplicationError(MessageFactory.system_edituser_success, ApplicationError.NO_ERROR);
			retAE.addApplicationErrors(error);
		}
		else
		{
			retAE = start();
			retAE.setApplicationErrors(errorsAE);
		}		
				
		return retAE;		
	}
	
	public ApplicationEvent cancel(ApplicationEvent appEvent, Dictionary dto) throws ApplicationException
	{
		DisplayMainLayout useCase = new DisplayMainLayout(_as, new Dictionary());
		return useCase.start(); 
	}

	public ApplicationEvent cancel(ApplicationEvent appEvent) throws ApplicationException 
	{
		DisplayMainLayout useCase = new DisplayMainLayout(_as, new Dictionary());
		return useCase.start(); 
	}

	private Publicacion _buildPublication(Dictionary dto, ArrayList files)
	{
		Publicacion bean = null;
		
		String idUsuario = dto.getSingleValue(BusinessGlobals.DTO_FIELD_USER_ID);
		
		if (idUsuario != null)
		{
			bean = new Publicacion(null);
			bean.setIdUsuario(new Integer(idUsuario));
			SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
			try {
				bean.setFechaPublicacion(df.parse(dto.getSingleValue(BusinessGlobals.DTO_FIELD_PUBLICATION_FECHAPUBLICACION)));
			} catch (ParseException e) {
				e.printStackTrace();
			}
			bean.setTexto(dto.getSingleValue(BusinessGlobals.DTO_FIELD_PUBLICATION_TEXTO));
			//String ficheroImagen = dto.getSingleValue(BusinessGlobals.DTO_FIELD_PUBLICATION_FICHEROIMAGEN);
			bean.setImagen(_getImagen(files));
			bean.setLink(dto.getSingleValue(BusinessGlobals.DTO_FIELD_PUBLICATION_LINK));
			if (dto.getSingleValue(BusinessGlobals.DTO_FIELD_PUBLICATION_ZONA)!= null)
				bean.setIdZona(new Integer(dto.getSingleValue(BusinessGlobals.DTO_FIELD_PUBLICATION_ZONA)));
			if (dto.getSingleValue(BusinessGlobals.DTO_FIELD_PUBLICATION_BARRIO)!= null)
				bean.setIdBarrio(new Integer(dto.getSingleValue(BusinessGlobals.DTO_FIELD_PUBLICATION_BARRIO)));
			bean.setDireccion(dto.getSingleValue(BusinessGlobals.DTO_FIELD_PUBLICATION_DIRECCION));
		}
		
		return bean;
	}
	
	private Blob _getImagen (ArrayList files) 
	{
		Blob blob = null;
	    
		for(int i=0;i<files.size();i++)
	    {
	        File file = (File) files.get(i);
		
			if (file.isFile())
			{
				byte[] fileContent = new byte[(int)file.length()];
				try {
					FileInputStream fin = new FileInputStream(file);
					fin.read(fileContent);
					
					blob = new SerialBlob(fileContent);
				} catch (FileNotFoundException e1) {
					e1.printStackTrace();
				} catch (SQLException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				}        
			}
	    }

		return blob;
    }
	
}
