package usecase;
 
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import control.factory.MessageFactory;
import control.interfaces.DataEntryUseCase;
import control.interfaces.UseCase;
import control.utility.ApplicationError;
import control.utility.ApplicationEvent;
import control.utility.ApplicationException;
import control.utility.ApplicationGlobals;
import control.utility.ApplicationStore;
import control.utility.BusinessGlobals;
import control.utility.JSPDispatcher;
import control.utility.Logger;
import control.utility.Utility;
import database.DatabaseDaoDbUtility;
import database.hibernate.SystemHibernateDaoDbUtility;
import database.utility.HibernateUtil;
import entity.Dictionary;
import entity.Envelope;
import entity.hibernate.Usuario;

/*
 * **************************************************************************
 * <p>EditarUsuario use case class
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */

public class RegistrarUsuario implements UseCase, DataEntryUseCase
{
	private ApplicationStore _as = null;

	private Dictionary _editedUser = null;
	
	public RegistrarUsuario(ApplicationStore as)
	{
		_as = as;
	}
	
	public RegistrarUsuario(ApplicationStore as, Dictionary editedUser)
	{
		_as = as;
		_editedUser = editedUser;
	}
	
	public ApplicationEvent start() throws ApplicationException
	{
		ApplicationEvent retAE = new ApplicationEvent();
		 
		//recover user data if updating user...
		
		Object [] output = new Object[1];
		
/*		if (_editedUser != null)
		{
			//i am editing an existing user int the system...
			
			String userId = _editedUser.getSingleValue(ApplicationGlobals.SYSTEM_PARAM_ID_EVENT_PARAMETERS);

			//retrieve user data from Prototype...
//			Usuario  dbUser = Prototype.retrieveUser(userId);
			
			//retrieve user data from persistence hibernate...
//			Usuario  dbUser = SystemHibernateDaoDbUtility.retrieveUser(userId);
			
			//retrieve user data from persistence...
			DatabaseDaoDbUtility systemDaoDbUtility = new DatabaseDaoDbUtility();
			Usuario dbUser = systemDaoDbUtility.retrieveUser(userId);
			
			if (dbUser == null)
			{
				retAE.setActiveUseCase(_as.getEvent().getActiveUseCase());		
				ApplicationError error [] = new ApplicationError[1];
				error[0] = new ApplicationError(MessageFactory.system_edituser_error_unknownUser, ApplicationError.USER_PASSWORD_INCORRECT);
				retAE.addApplicationErrors(error);
			
				return retAE;
			}
			
			output[0] = dbUser;
		}
		else
		{*/
			//i am adding a new user into the system...
			
			Usuario newUser = new Usuario();
			newUser.setIdUsuario(null);
			output[0] = newUser;
//		}
		
		//build dto & store it in application event...
		//create envelope to jsp & redispatch...
		//envelope contains array with: [0]=user...
		//set default window & layout...
		
		Envelope envelope = new Envelope(output);		
				
		retAE.setTarget(JSPDispatcher.getTarget(JSPDispatcher.RegistrarUsuario));	
		retAE.setDataModel(envelope);		
		retAE.setActiveUseCase(this);
		
		return retAE;				
	}

	public ApplicationEvent accept(ApplicationEvent appEvent, Dictionary dto) throws ApplicationException 
	{
		ApplicationEvent retAE = new ApplicationEvent();
		boolean consolidated  = false;
		Usuario bean = null; 
		
		//set default window & layout...
		
		retAE.setTarget(JSPDispatcher.getTarget(JSPDispatcher.RegistrarUsuario));
		
	    //build bean from dto...
		bean = _buildUser(dto);
	
		//retrieve user data from Prototype...
//		Usuario  dbUser = Prototype.retrieveUser(bean.getEmail());
		
		//retrieve user data from persistence hibernate...
//		Usuario  dbUser = SystemHibernateDaoDbUtility.retrieveUser(bean.getEmail());
		
		//retrieve user data from persistence...
		DatabaseDaoDbUtility systemDaoDbUtility = new DatabaseDaoDbUtility();
		Usuario dbUser = systemDaoDbUtility.retrieveUser(bean.getEmail());
		
		//insert usuario data from persistence hibernate...		 
//		if (dbUser == null && SystemHibernateDaoDbUtility.insertUsuario(bean)) {
		
		//insert usuario data from persistence...				
		if (dbUser == null && systemDaoDbUtility.insertUsuario(bean)){
			dbUser = systemDaoDbUtility.retrieveUser(bean.getEmail());
			consolidated = true;
		}
		else
		{
			ApplicationError error [] = new ApplicationError[1];
			error[0] = new ApplicationError(MessageFactory.system_edituser_error_persistence, ApplicationError.INTERNAL_APPLICATION_ERROR);
			retAE.addApplicationErrors(error);
		}
     	ArrayList errorsAE = retAE.getApplicationErrors();
 		
		if (consolidated)
		{
			Utility.setLoggedUser(_as, dbUser);
			
			Logger.log("User " + dbUser.getIdUsuario() + " logged succesfully");			
			
			//show wellcome page...

			Welcome welcome = new Welcome(_as,null);
			return welcome.start();
		}
		else
		{
			retAE = start();
			retAE.setApplicationErrors(errorsAE);
		}		
				
		return retAE;		
	}
	
	public ApplicationEvent cancel(ApplicationEvent appEvent, Dictionary dto) throws ApplicationException
	{
		AccesoUsuario useCase = new AccesoUsuario(_as, null);
		return useCase.start(); 
	}

	public ApplicationEvent cancel(ApplicationEvent appEvent) throws ApplicationException 
	{
		AccesoUsuario useCase = new AccesoUsuario(_as, null);
		return useCase.start(); 
	}

	private Usuario _buildUser(Dictionary dto)
	{
		Usuario bean = null;
		
		String key = dto.getSingleValue(BusinessGlobals.DTO_FIELD_USER_ID);
		
		if (key != null)
		{
			bean = new Usuario();
			bean.setIdUsuario(null);
			bean.setPasword(dto.getSingleValue(BusinessGlobals.DTO_FIELD_USER_PWD));
			bean.setTipoUsuario(dto.getSingleValue(BusinessGlobals.DTO_FIELD_USER_TIPOUSUARIO));			
			bean.setNombre(dto.getSingleValue(BusinessGlobals.DTO_FIELD_USER_NOMBRE));
			bean.setApellidos(dto.getSingleValue(BusinessGlobals.DTO_FIELD_USER_APELLIDOS));
			String fechaNacimiento = dto.getSingleValue(BusinessGlobals.DTO_FIELD_USER_FECHANACIMIENTO);
			if (fechaNacimiento!=null && !fechaNacimiento.equals("")) {
				try {
					SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
					bean.setFechaNacimiento(df.parse(fechaNacimiento));
				} catch (ParseException e) {
					e.printStackTrace();
				}
			}
			bean.setDireccion(dto.getSingleValue(BusinessGlobals.DTO_FIELD_USER_DIRECCION));
			bean.setCodigoPostal(dto.getSingleValue(BusinessGlobals.DTO_FIELD_USER_CODIGOPOSTAL));
			bean.setPais(dto.getSingleValue(BusinessGlobals.DTO_FIELD_USER_PAIS));
			bean.setEmail(dto.getSingleValue(BusinessGlobals.DTO_FIELD_USER_EMAIL));
		}
		
		return bean;
	}
}
