package usecase;
 
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import control.factory.MessageFactory;
import control.interfaces.DataEntryUseCase;
import control.interfaces.UseCase;
import control.utility.ApplicationError;
import control.utility.ApplicationEvent;
import control.utility.ApplicationException;
import control.utility.ApplicationGlobals;
import control.utility.ApplicationStore;
import control.utility.BusinessGlobals;
import control.utility.JSPDispatcher;
import control.utility.Logger;
import control.utility.Utility;
import database.DatabaseDaoDbUtility;
import database.hibernate.SystemHibernateDaoDbUtility;
import database.utility.HibernateUtil;
import entity.Dictionary;
import entity.Envelope;
import entity.hibernate.Usuario;


/*
 * **************************************************************************
 * <p>SeguirUsuario use case class
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */

public class SeguirUsuario implements UseCase, DataEntryUseCase
{
	private ApplicationStore _as = null;
	
	public SeguirUsuario(ApplicationStore as, Dictionary dicc)
	{
		_as = as;
	}
	
	public SeguirUsuario(ApplicationStore as)
	{
		_as = as;
	}
	
	public ApplicationEvent start() throws ApplicationException
	{
		ApplicationEvent retAE = new ApplicationEvent();
		 
		Object [] output = new Object[3];

		//i am adding a new seguimiento usuario into the system...
			
		Usuario user = Utility.getLoggedUser(_as);
		output[0] = user;
		
		//retrieve seguimientos data from Prototype...
//		List seguimientos = Prototype.retrieveSeguimientos(user.getIdUsuario());

		//retrieve seguimientos data from persistence hibernate...
//		List seguimientos = SystemHibernateDaoDbUtility.retrieveSeguimientos(user.getIdUsuario());

		//retrieve seguimientos data from persistence...
		DatabaseDaoDbUtility systemDaoDbUtility = new DatabaseDaoDbUtility();
		List seguimientos = systemDaoDbUtility.retrieveSeguimientos(user.getIdUsuario());
		output[1] = seguimientos;
		
		//retrieve usuarios data from Prototype...
//		List usuarios = Prototype.retrieveUsuarios();
		
		//retrieve usuarios data from persistence hibernate...
//		List usuarios = SystemHibernateDaoDbUtility.retrieveUsuarios();
		
		//retrieve usuarios data from persistence...
		List usuarios = systemDaoDbUtility.retrieveUsuarios();
		output[2] = usuarios;
		
		//build dto & store it in application event...
		//create envelope to jsp & redispatch...
		//envelope contains array with: [0]=publication, [1]=comments...
		//set default window & layout...
		
		Envelope envelope = new Envelope(output);		
				
		retAE.setTarget(JSPDispatcher.getTarget(JSPDispatcher.SeguimientoUsuarios));	
		retAE.setDataModel(envelope);		
		retAE.setActiveUseCase(this);		
			
		return retAE;				
	}

	public ApplicationEvent accept(ApplicationEvent appEvent, Dictionary dto) throws ApplicationException 
	{
		ApplicationEvent retAE = new ApplicationEvent();
		boolean consolidated  = false;
		List bean = null; 
		
		retAE.setTarget(JSPDispatcher.getTarget(JSPDispatcher.SeguimientoUsuarios));
		
	    //build bean from dto...
		bean = _buildUsuarios(dto);
		
		Usuario user = Utility.getLoggedUser(_as);
		
		//insert seguimientos data from persistence hibernate...		 
//		if (SystemHibernateDaoDbUtility.insertSeguimientos(user, bean)) {
		
		//insert seguimientos data from persistence...		
		DatabaseDaoDbUtility systemDaoDbUtility = new DatabaseDaoDbUtility();
		if (systemDaoDbUtility.insertSeguimientos(user, bean)){
			consolidated = true;
		}
		else
		{
			ApplicationError error [] = new ApplicationError[1];
			error[0] = new ApplicationError(MessageFactory.system_edituser_error_persistence, ApplicationError.INTERNAL_APPLICATION_ERROR);
			retAE.addApplicationErrors(error);
		}
		
     	ArrayList errorsAE = retAE.getApplicationErrors();
 		
		if (consolidated)
		{
			DisplayMainLayout usecase = new DisplayMainLayout(_as, new Dictionary());
			retAE = usecase.start();
			
			ApplicationError error [] = new ApplicationError[1];
			error[0] = new ApplicationError(MessageFactory.system_edituser_success, ApplicationError.NO_ERROR);
			retAE.addApplicationErrors(error);
		}
		else
		{
			retAE = start();
			retAE.setApplicationErrors(errorsAE);
		}		
				
		return retAE;		
	}
	
	public ApplicationEvent cancel(ApplicationEvent appEvent, Dictionary dto) throws ApplicationException
	{
		DisplayMainLayout useCase = new DisplayMainLayout(_as, new Dictionary());
		return useCase.start(); 
	}

	public ApplicationEvent cancel(ApplicationEvent appEvent) throws ApplicationException 
	{
		DisplayMainLayout useCase = new DisplayMainLayout(_as, new Dictionary());
		return useCase.start(); 
	}

	private List _buildUsuarios(Dictionary dto)
	{
		List bean = null;
		
		String[] listUsuario = dto.getMultipleValues(BusinessGlobals.DTO_FIELD_USER_ID);
		
		if (listUsuario != null)
		{
			bean = new ArrayList();
			for (int i = 0; i<listUsuario.length; i++)
			bean.add(listUsuario[i]);
		}
		
		return bean;
	}
	
}
