package control.display;

/*
 * **************************************************************************
 * @(#)DisplayImageFile.java	11/2012
 * --------------------------------------------------------------------------
 * Copyright (c) 2012 T-Systems Iberia. All Rights Reserved.
 * **************************************************************************
 */

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jfree.chart.servlet.ServletUtilities;

/*
 * **************************************************************************
 *  DisplayImageFile
 *
 * --------------------------------------------------------------------------
 * @version     0.1 11/2012
 * @author      igil
 * --------------------------------------------------------------------------
 * **************************************************************************
 */

public class DisplayImageFile extends HttpServlet {

	public final static long serialVersionUID = 1L;
	
	public DisplayImageFile() {
		super();
	}

	public void init() throws ServletException {
		return;
	}

	public void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		String filename = request.getParameter("filename");

		if (filename == null) {
			throw new ServletException("Parameter 'filename' must be supplied");
		}

		//replace ".." with ""
		//this is to prevent access to the rest of the file system
		filename = ServletUtilities.searchReplace(filename, "..", "");

		//check the file exists
		File file = new File(System.getProperty("java.io.tmpdir"), filename);
		if (!file.exists()) {
			throw new ServletException("File '" + file.getAbsolutePath() + "' does not exist");
		}

		//serve it up
		sendTempFile(file, response);

		//delete temp file
		file.delete();
		
		return;
	}

	public static void sendTempFile(File file, HttpServletResponse response)
			throws IOException {

		String mimeType = null;
		String filename = file.getName();
		if (filename.length() > 5) {
			if (filename.substring(filename.length() - 5, filename.length()) .equals(".jpeg")) {
				mimeType = "image/jpeg";
			} else if (filename.substring(filename.length() - 4, filename.length()).equals(".jpg")) {
				mimeType = "image/jpeg";
			} else if (filename.substring(filename.length() - 4, filename.length()).equals(".png")) {
				mimeType = "image/png";
			} else if (filename.substring(filename.length() - 4, filename.length()).equals(".gif")) {
				mimeType = "image/gif";
			}
		}
		sendTempFile(file, response, mimeType);
	}

	public static void sendTempFile(File file, HttpServletResponse response, String mimeType) throws IOException {

		if (file.exists()) {
			BufferedInputStream bis = new BufferedInputStream(
					new FileInputStream(file));

			//set HTTP headers
			if (mimeType != null) {
				response.setHeader("Content-Type", mimeType);
			}
			response.setHeader("Content-Length", String.valueOf(file.length()));
			SimpleDateFormat sdf = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss z", Locale.ENGLISH);
			sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
			response.setHeader("Last-Modified", sdf.format(new Date(file.lastModified())));

			BufferedOutputStream bos = new BufferedOutputStream(response.getOutputStream());
			
			byte[] input = new byte[1024];
			boolean eof = false;
			
			while (!eof) {
				int length = bis.read(input);
				if (length == -1) {
					eof = true;
				} else {
					bos.write(input, 0, length);
				}
			}
			bos.flush();
			bis.close();
			bos.close();
		} else {
			throw new FileNotFoundException(file.getAbsolutePath());
		}
		return;
	}
}
