package control.navigation;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.SocketException;
import java.util.ArrayList;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import usecase.AccesoUsuario;
import usecase.Logout;

import com.oreilly.servlet.ServletUtils;

import control.factory.RequestFactory;
import control.factory.UseCaseFactory;
import control.interfaces.ActivableUseCase;
import control.interfaces.DataEntryUseCase;
import control.interfaces.DataEntryWithFilesUseCase;
import control.interfaces.DownloadFileUseCase;
import control.interfaces.FiltrableUseCase;
import control.interfaces.NavegableUseCase;
import control.interfaces.PaginableUseCase;
import control.interfaces.SelectionFromListUseCase;
import control.interfaces.SingleRowMaintenanceUseCase;
import control.interfaces.SortableUseCase;
import control.interfaces.TabbedUseCase;
import control.interfaces.UseCase;
import control.utility.ApplicationEvent;
import control.utility.ApplicationException;
import control.utility.ApplicationGlobals;
import control.utility.ApplicationStore;
import control.utility.FileHelper;
import control.utility.Logger;
import entity.Dictionary;
import entity.Envelope;

/*
 * **************************************************************************
 *  <p>EventProcessor class
 *
 * --------------------------------------------------------------------------
 * @version     0.1 04/2013
 * @author      Fina Cara
 * --------------------------------------------------------------------------
 * **************************************************************************
 */

public class EventProcessor 
{
	private ApplicationStore _appStore = null;

	private HttpServletRequest _request = null;

	private HttpServletResponse _response = null;
	
	public EventProcessor(ApplicationStore as, HttpServletRequest request, HttpServletResponse response) {
		super();
		_appStore = as;
		_request = request;
		_response = response;
	}
	
	public void process(){
		ApplicationEvent ae = (ApplicationEvent) _appStore.getEvent();
                		if (ae != null) {
			
			//Clean old errors saved in event...
			ae.cleanErrors();
			
			//Make a dictionary with request values...
			RequestFactory factory = new RequestFactory(_request);
			Dictionary dicc = factory.makeDictionary();
			 
			//Process event...
			if (!_processMenuEvents(dicc)){
				if (!_processStandardEvents(dicc)){
					if (!_processFileEvents(dicc)){
						_processCustomEvents(dicc);
					}
				}
			}
		}
	}

	private boolean _processMenuEvents(Dictionary dicc) {
		
		String eventType = dicc.getSingleValue(ApplicationGlobals.SYSTEM_PARAM_ID_EVENT);

		if (ApplicationGlobals.SYSTEM_APPLICATION_EVENT_MENU_NAVIGATE.equals(eventType)) {
									
			String useCaseString = dicc.getSingleValue(ApplicationGlobals.SYSTEM_PARAM_ID_EVENT_USECASE);
			
			if (useCaseString != null  &&  useCaseString.trim().length() > 0)
			{
				UseCase useCase = UseCaseFactory.makeUseCase(_appStore,(dicc!=null)?dicc:new Dictionary(),useCaseString);
				if (useCase!=null){
					ApplicationEvent ae;
					try {
						ae = useCase.start();
						_appStore.setEvent(ae);
					} catch (ApplicationException e) {
						e.printStackTrace();
					}					
				}
				else{
					Logger.log("APARBAR >> EventProcessor >> processMenuEvent >> Error make instance of "+useCaseString);
				}
			}
			
			return true;
		}
		
		return false;
	}

	private boolean _processStandardEvents(Dictionary dicc) {
		
		ApplicationEvent ae = null;
		String eventType = dicc.getSingleValue(ApplicationGlobals.SYSTEM_PARAM_ID_EVENT);
		String eventParameter = dicc.getSingleValue(ApplicationGlobals.SYSTEM_PARAM_ID_EVENT_PARAMETERS);
		
		if (eventType == null) { return false;	}
	
		//Try process the event as standard event...
		if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_ACCEPT)) {	ae = _accept(dicc); }			
		else if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_CANCEL)) { ae = _cancel(dicc); }
		else if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_LOGOUT)) { ae = _logout(dicc); }
		else if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_LOGIN)) { ae = _login(dicc); }
		else if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_NAVIGATE)) { ae = _navigation(dicc); }		
		else if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_PAGINATION_ACTION)) {	ae=_processPaginationEvent(dicc,eventParameter); }
		else if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_CREATE)) { ae = _create(dicc); }	
		else if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_DELETE)) { ae = _delete(dicc); }			
		else if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_EDIT)) { ae = _edit(dicc); }
		else if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_ITEM_SELECTION)) { ae = _select(dicc); }
		else if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_FILTER_TABLE)) { ae = _filter(dicc); }					
		else if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_SORT_ASCENDING)) { ae = _sortAscending(dicc); }
		else if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_SORT_DESCENDING)) { ae = _sortDescending(dicc); }							
		else if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_CHANGE_TAB)){	ae = _changeTab(dicc); }
		else if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_ACTIVATE)){	ae = _activate(dicc); }
		else if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_DEACTIVATE)){	ae = _deactivate(dicc); }
		
		//It isn't a standardevent...
		else return false;
		
		//It was a standard event. Save updated event in session...
		_appStore.setEvent(ae);
		return true;
		 
	}
	
	private boolean _processFileEvents(Dictionary dicc){
		ApplicationEvent ae = null;
		String [] eventTypes = _request.getParameterValues(ApplicationGlobals.SYSTEM_PARAM_ID_EVENT);
		
		if (eventTypes != null)
		{
			String eventType = eventTypes[0];
			if (eventType != null)
			{
				if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_ACCEPT_WITH_FILES)) 
				{
					ae = _acceptWithFiles(dicc);
					_appStore.setEvent(ae);
					return true;
				}else if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_DOWNLOAD_FILE)){
					ae = _downloadFile(dicc);
					ae.setTarget(null);
					_appStore.setEvent(ae);
					return true;
				}else if (eventType.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_UPLOAD_FILE)){
					ae = _uploadFileUseCase(dicc);
					ae.setTarget(null);
					_appStore.setEvent(ae);
					return true;
				}
			}

		}
		return false;
	}
	
	private ApplicationEvent _processPaginationEvent(Dictionary dicc, String eventParameter){
		ApplicationEvent ae = null;
		if (eventParameter.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_FIRST_PAGE)) { ae = _firstPage(dicc); } 
		else if (eventParameter.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_LAST_PAGE)) { ae = _lastPage(dicc); } 
		else if (eventParameter.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_NEXT_PAGE)) { ae = _nextPage(dicc); } 
		else if (eventParameter.equals(ApplicationGlobals.SYSTEM_APPLICATION_EVENT_PREVIOUS_PAGE)) { ae = _previousPage(dicc);	}
		return ae;
	}

	private ApplicationEvent _sortDescending(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		SortableUseCase useCase = (SortableUseCase) _getUseCase(dicc);
		
		try {  return useCase.sort(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}

	private ApplicationEvent _sortAscending(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		SortableUseCase useCase = (SortableUseCase) _getUseCase(dicc);
		
		try {  return useCase.sort(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}

	private ApplicationEvent _accept(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		DataEntryUseCase useCase = (DataEntryUseCase) _getUseCase(dicc);
		
		try {  return useCase.accept(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}

	private ApplicationEvent _cancel(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		DataEntryUseCase useCase = (DataEntryUseCase) _getUseCase(dicc);
		
		try {  return useCase.cancel(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}
	
	private ApplicationEvent _logout(Dictionary dicc) {
		Logout logout = new Logout(_appStore,dicc);
		
		try {  return logout.start();  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}
	
	private ApplicationEvent _login(Dictionary dicc) {
		AccesoUsuario accesoUsuario = new AccesoUsuario(_appStore,dicc);
		
		try {  return accesoUsuario.start();  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}

	private ApplicationEvent _firstPage(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		PaginableUseCase useCase = (PaginableUseCase) _getUseCase(dicc);
		
		try {  return useCase.first(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}

	private ApplicationEvent _select(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		SelectionFromListUseCase useCase = (SelectionFromListUseCase) _getUseCase(dicc);
		
		try {  return useCase.entrySelected(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}

	private ApplicationEvent _filter(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		FiltrableUseCase useCase = (FiltrableUseCase) _getUseCase(dicc);

		try {  return useCase.filter(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}
	
	private ApplicationEvent _edit(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		SingleRowMaintenanceUseCase useCase = (SingleRowMaintenanceUseCase) _getUseCase(dicc);
		
		try {  return useCase.edit(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}

	private ApplicationEvent _delete(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		SingleRowMaintenanceUseCase useCase = (SingleRowMaintenanceUseCase) _getUseCase(dicc);
		
		try {  return useCase.delete(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}

	private ApplicationEvent _create(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		SingleRowMaintenanceUseCase useCase = (SingleRowMaintenanceUseCase) _getUseCase(dicc);
		
		try {  return useCase.add(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}

	private ApplicationEvent _lastPage(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		PaginableUseCase useCase = (PaginableUseCase) _getUseCase(dicc);
		
		try {  return useCase.last(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}

	private ApplicationEvent _nextPage(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		PaginableUseCase useCase = (PaginableUseCase) _getUseCase(dicc);
		
		try {  return useCase.next(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}

	private ApplicationEvent _previousPage(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		PaginableUseCase useCase = (PaginableUseCase) _getUseCase(dicc);
		
		try {  return useCase.previous(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}
	
	private ApplicationEvent _changeTab(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		TabbedUseCase useCase = (TabbedUseCase) _getUseCase(dicc);
		
		try {  return useCase.changeTab(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}
	
	private ApplicationEvent _deactivate(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		ActivableUseCase useCase = (ActivableUseCase) _getUseCase(dicc);
		
		try {  return useCase.deactivate(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}
	
	private ApplicationEvent _activate(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		ActivableUseCase useCase = (ActivableUseCase) _getUseCase(dicc);
		
		try {  return useCase.activate(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}
	
	private ApplicationEvent _downloadFile(Dictionary dicc) {
		
		ApplicationEvent ae = _appStore.getEvent();
		DownloadFileUseCase useCase = (DownloadFileUseCase) _getUseCase(dicc);
		try
		{
			//export data...
			
			ae = useCase.download(ae, dicc);
		
			ArrayList errors = ae.getApplicationErrors();
			if (errors != null  &&  errors.size() == 0)
			{
				Envelope dto = (Envelope)ae.getDto();
				
				if (dto != null)
				{
					File theResultFile = (File) dto.getContent();
			
					if (theResultFile != null)
					{
						String fileDown = "./" + theResultFile.getName();
				
						String contentType = null;//_appStore.getSession().getServletContext().getMimeType(fileDown);
						if (contentType == null)
						{
							contentType = FileHelper.getMimeType(theResultFile);
						}
		
						_response.setContentType("application/x-download");
						_response.setHeader("Content-Disposition","attachment; filename=\""+ theResultFile.getName() + "\"");
					
						//descargamos el fichero....
				
						ServletOutputStream out = _response.getOutputStream();
						ServletUtils.returnFile(fileDown,out);
						out.flush();
						out.close();
				
						//borramos el fichero temporal....
						theResultFile.delete();
					}
				}
			}
		}
		catch(ApplicationException e)
		{
			System.out.println(e.getMessage());
		}
		catch(FileNotFoundException e)
		{
			System.out.println(e.getMessage());
		}
		catch(SocketException e)
		{
			//Download canceled by user...
		}
		catch(IOException e)
		{
			System.out.println(e.getMessage());
		}
		
		return ae;
	}

	private ApplicationEvent _navigation(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		NavegableUseCase useCase = (NavegableUseCase) _getUseCase(dicc);
		
		try {  return useCase.navigate(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}
	
	private ApplicationEvent _uploadFileUseCase(Dictionary dicc) {
		ApplicationEvent ae = _appStore.getEvent();
		NavegableUseCase useCase = (NavegableUseCase) _getUseCase(dicc);
		
		try {  return useCase.navigate(ae, dicc);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}

	private ApplicationEvent _acceptWithFiles(Dictionary dicc) {
		
		ApplicationEvent ae = _appStore.getEvent();
		RequestFactory rf = new RequestFactory(_request);
		
		Dictionary multipartDiccionary = rf.makeDictionary(true);
		ArrayList multipartFiles = rf.makeFiles();
		
		DataEntryWithFilesUseCase useCase = (DataEntryWithFilesUseCase) _getUseCase(dicc);
		if (useCase == null)
		{
			useCase = (DataEntryWithFilesUseCase) _getUseCase(dicc);
		}
		
		//add Dictionary data...
		String parameter = dicc.getSingleValue(ApplicationGlobals.SYSTEM_PARAM_ID_EVENT_PARAMETERS);
		String extraParameter = dicc.getSingleValue(ApplicationGlobals.SYSTEM_PARAM_ID_EVENT_EXTRA_PARAMETERS);
		
		if (parameter!=null) multipartDiccionary.setValue(ApplicationGlobals.SYSTEM_PARAM_ID_EVENT_PARAMETERS,parameter);
		if (extraParameter!=null) multipartDiccionary.setValue(ApplicationGlobals.SYSTEM_PARAM_ID_EVENT_EXTRA_PARAMETERS,extraParameter);
		
		try {  return useCase.acceptWithFiles(ae, multipartDiccionary, multipartFiles);  }
		catch(ApplicationException exc)  {  }
		
		return null;
	}

	private boolean _processCustomEvents(Dictionary dicc) {
		String eventType = dicc.getSingleValue(ApplicationGlobals.SYSTEM_PARAM_ID_EVENT);
		if (eventType==null){
			//there isn't event
			//go to login frame...
			_appStore.setEvent(_login(dicc));
		}
		return true;
	}
	
	private UseCase _getUseCase(Dictionary dicc)
	{
		UseCase activeUseCase = _appStore.getEvent().getActiveUseCase();
		
		String eventUseCase = dicc.getSingleValue(ApplicationGlobals.SYSTEM_PARAM_ID_EVENT_USECASE);
		if (activeUseCase.getClass().getName().equals(eventUseCase)){
			return activeUseCase;
		}
		else{
			activeUseCase = UseCaseFactory.makeUseCase(_appStore,(dicc!=null)?dicc:new Dictionary(),eventUseCase);
			try {
				activeUseCase.start();
			} catch (ApplicationException e) {
				e.printStackTrace();
			}
		}
		return activeUseCase;
	}

}
